/*****************************************************************************
* @file     TestCode_ADC_Interrupt.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_ADC_Interrupt.h"
#include "main_conf.h"

#define AVDD (3.3)		// FPGA Reference Voltage is 3.3V
#define RES (4096)		// 2^12 = 4096 bits
#define LSB (AVDD/RES)  // 1LSB = VDD/RES = 3.3/4096 = 805.664mV

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint32_t MilliSec;
static uint32_t ADCData[8];
static ADC_CFG_Type ADC_Config;

static Bool DoneFlag;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static uint32_t Main_Average8Data(uint32_t *pData);

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_ADC_Interrupt(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

/*******************************************************************************
* @brief      ADC interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void ADC_IRQHandler_ADC_Interrupt(void) {
	ADC_STATUS_Type status; // check interrupt flag

	status = HAL_ADC_GetStatus(ADC);
	
	// EOC Flag	
	if (status & ADC_STATUS_SINGLE_INT) {
       	DoneFlag = TRUE;
		HAL_ADC_ClearStatus(ADC, ADC_STATUS_SINGLE_INT);
	}
}

/*******************************************************************************
* @brief      Main_AdcInterruptTest
* @param      None
* @return     None
*******************************************************************************/
void Main_AdcInterruptTest(void) {
	uint32_t i;
	uint32_t j;
	uint32_t Average;
	uint32_t AdcCh;
	uint32_t AdcSq;
	float volt = 0.0;
//	uint16_t volt_dec; 
//	uint16_t volt_float;
	   
	// Set ADC input pin
	HAL_ADC_PortInit(0);
	HAL_ADC_PortInit(1);
	HAL_ADC_PortInit(2);
	HAL_ADC_PortInit(3);
	HAL_ADC_PortInit(4);
	HAL_ADC_PortInit(5);
	HAL_ADC_PortInit(6);
	HAL_ADC_PortInit(7);
//	HAL_ADC_PortInit(8);    // Excluded this channel for in use as USART10 debug port
//	HAL_ADC_PortInit(9);    // Excluded this channel for in use as USART10 debug port
	HAL_ADC_PortInit(10);
	HAL_ADC_PortInit(11);
	HAL_ADC_PortInit(12);
	HAL_ADC_PortInit(13);
	HAL_ADC_PortInit(14);
	HAL_ADC_PortInit(15);   // Excluded this channel for in use as UART1 debug port
	HAL_ADC_PortInit(16);   // Excluded this channel for in use as UART1 debug port
	HAL_ADC_PortInit(17);
	
	// Set ADC external clock
	HAL_SCU_MiscClockConfig(7, ADC_TYPE, CLKSRC_MCLK, 12); // 48MHz/12 = 4MHz

	// Initialize and configure ADC
	ADC_Config.TriggerInfo = DISABLE;
	ADC_Config.ChannelInfo = ENABLE;
	ADC_Config.Dma = DISABLE;
	ADC_Config.Restart = DISABLE;

	ADC_Config.Mode = ADC_MODE_SINGLE;
	ADC_Config.TriggerSelection = ADC_TRIGGER_SEL_OLNY_SOFT;

	ADC_Config.SamplingTime = 1;
	ADC_Config.SequenceCounter = ADC_SEQ_NUM_1;

	ADC_Config.ExternalClock = ENABLE;
	ADC_Config.InternalClockDivider = 1;

	HAL_ADC_Init(ADC, &ADC_Config);
	HAL_ADC_ConfigureInterrupt(ADC, ADC_INTERRUPT_SINGLE, ENABLE);

	// Enable SysTick and ADC interrupt
	SysTick_Config(SystemCoreClock / 1000);

	NVIC_SetPriority(ADC_IRQn, 3);
	NVIC_EnableIRQ(ADC_IRQn);

	__enable_irq();

	// Start ADC test
	while(1) {
		for (i = 0; i < 21; i++) {
			HAL_ADC_ConfigureOneSequence(ADC, ADC_SEQUENCE_0, (ADC_CHANNEL_Type)(ADC_CHANNEL_0 + i));
			for(j = 0; j < 8; j++) {
				DoneFlag = FALSE;
				HAL_ADC_Start(ADC);
				while(DoneFlag == FALSE);
				ADCData[j] = HAL_ADC_GetConversionData(ADC, ADC_DATA_SEL_LAST);
			}
		
			cputs("--------------------------------------------------\r\n");
			cprintf("Selected Channel: %d\r\n", i);
		
			Average = Main_Average8Data(ADCData);
			volt = Average*LSB;                       
			cprintf("ADC Value: %04X\r\n", Average);
            cprintf("Voltage (Calc): %.03f\r\n", volt);
            
//			volt_dec = (uint16_t)volt;
//			volt_float = (uint16_t)((volt-volt_dec)*1000);            
//			cprintf("Voltage (Calc): %d.%03d\r\n", volt_dec, volt_float);
			
			AdcCh = (ADC->CSCR&ADC_CSCR_CACH_Msk);
			cprintf("Current Channel: %d\r\n", AdcCh);
			
			AdcSq = (ADC->CSCR&ADC_CSCR_CSEQN_Msk);
			cprintf("Current Sequence is AD: %d\r\n", AdcSq);
            
            // 1 second delay
            MilliSec = 1000;
            while(MilliSec);
		}
	}	
}

/*******************************************************************************
* @brief      Average of N Data
* @param      pData : Pointer contains N data
* @return     Average value
*******************************************************************************/
static uint32_t Main_Average8Data(uint32_t *pData) {
	uint32_t i;
	uint32_t Average;
	
	Average = 0;
	for (i = 0; i < 8; i++) {
		Average += *(pData++);
	}
	Average >>= 3;
	
	return Average;
}
/* --------------------------------- End Of File ------------------------------ */
