/*****************************************************************************
* @file     TestCode_ADC_Secuqence.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_ADC_Sequence.h"
#include "main_conf.h"
/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint32_t MilliSec;
static ADC_CFG_Type ADC_Config;
static TIMER1n_CFG_Type TIMER1n_Config;
static Bool DoneFlag;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static void Main_InitializeTimer1n(void);

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_ADC_Sequence(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

/*******************************************************************************
* @brief      ADC interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void ADC_IRQHandler_ADC_Sequence(void) {
	ADC_STATUS_Type status; // check interrupt flag

	status = HAL_ADC_GetStatus(ADC);

	DoneFlag = TRUE;

	// EOC Flag
	if (status & ADC_STATUS_SINGLE_INT) {
		HAL_ADC_ClearStatus(ADC, ADC_STATUS_SINGLE_INT);
	}
	// EOS Flag
	if (status & ADC_STATUS_SEQUENCE_INT) {
		HAL_ADC_ClearStatus(ADC, ADC_STATUS_SEQUENCE_INT);
	}
	// Trigger Flag
	if (status & ADC_STATUS_TRIGGER_INT) {
		HAL_ADC_ClearStatus(ADC, ADC_STATUS_TRIGGER_INT);
	}
}

/*******************************************************************************
* @brief      Timer10 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER10_IRQHandler_ADC_Sequence(void) {
	if (HAL_TIMER1n_GetStatus(TIMER10) & TIMER1n_STATUS_MATCH_INT) {
		HAL_TIMER1n_ClearStatus(TIMER10, TIMER1n_STATUS_MATCH_INT);
	}
}

/*******************************************************************************
* @brief      Main_AdcSequenceTest
* @param      None
* @return     None
*******************************************************************************/
void Main_AdcSequenceTest(void) {
	uint32_t Data[4];
	ADC_TRIGGER_SOURCE_Type Source[4];
	ADC_CHANNEL_Type Channel[4]; 
    int i;
    
	// Initialize Timer1n
	Main_InitializeTimer1n();

	// Set ADC input pin
	HAL_ADC_PortInit(0);
	HAL_ADC_PortInit(1);
	HAL_ADC_PortInit(2);
	HAL_ADC_PortInit(3);

	// Set ADC external clock
	HAL_SCU_MiscClockConfig(7, ADC_TYPE, CLKSRC_MCLK, 12); // 48MHz/12 = 4MHz

	// Initialize and configure ADC
	ADC_Config.TriggerInfo = ENABLE;
	ADC_Config.ChannelInfo = ENABLE;
	ADC_Config.Dma = DISABLE;
	ADC_Config.Restart = DISABLE;
	ADC_Config.Mode = ADC_MODE_SEQUENTIAL;
	ADC_Config.TriggerSelection = ADC_TRIGGER_SEL_TIMER1n;
	ADC_Config.SamplingTime = 0x1F;
	ADC_Config.SequenceCounter = ADC_SEQ_NUM_4;
	ADC_Config.ExternalClock = ENABLE;
	ADC_Config.InternalClockDivider = 1;

	HAL_ADC_Init(ADC, &ADC_Config);
//	HAL_ADC_ConfigureInterrupt(ADC, ADC_INTERRUPT_SINGLE| ADC_INTERRUPT_SEQUENCE | ADC_INTERRUPT_TRIGGER, ENABLE);    
//	HAL_ADC_ConfigureInterrupt(ADC, ADC_INTERRUPT_SINGLE, ENABLE);
	HAL_ADC_ConfigureInterrupt(ADC, ADC_INTERRUPT_SEQUENCE, ENABLE); 
//	HAL_ADC_ConfigureInterrupt(ADC, ADC_INTERRUPT_TRIGGER, ENABLE);  
	Source[0] = ADC_TRIGGER_SOURCE_0;
	Source[1] = ADC_TRIGGER_SOURCE_0;
	Source[2] = ADC_TRIGGER_SOURCE_0;
	Source[3] = ADC_TRIGGER_SOURCE_0;
	HAL_ADC_ConfigureAllTrigger(ADC, 4, Source);
	Channel[0] = ADC_CHANNEL_0;
	Channel[1] = ADC_CHANNEL_1;
	Channel[2] = ADC_CHANNEL_2;
	Channel[3] = ADC_CHANNEL_3;
	HAL_ADC_ConfigureAllSequence(ADC, 4, Channel);

	// Enable SysTick and ADC interrupt
	SysTick_Config(SystemCoreClock / 1000);

	NVIC_SetPriority(ADC_IRQn, 3);
	NVIC_EnableIRQ(ADC_IRQn);

	__enable_irq();

	// Start timer
	DoneFlag = FALSE;
	HAL_TIMER1n_Start(TIMER10);
	
	while (1) {
		HAL_ADC_Start(ADC);

		while (DoneFlag == FALSE);
		DoneFlag = TRUE;

		Data[0] = HAL_ADC_GetConversionData(ADC, ADC_DATA_SEL_SEQ0);
		Data[1] = HAL_ADC_GetConversionData(ADC, ADC_DATA_SEL_SEQ1);
		Data[2] = HAL_ADC_GetConversionData(ADC, ADC_DATA_SEL_SEQ2);
		Data[3] = HAL_ADC_GetConversionData(ADC, ADC_DATA_SEL_SEQ3);
		
        for(i = 0 ; i < sizeof(Data)/sizeof(int) ; i++) {
            cprintf("AN%d = %04X\r\n", i, Data[i]);
        }

		// 1 second delay
		MilliSec = 1000;
		while (MilliSec);
	}
}

/*******************************************************************************
* @brief      Initialize timer1n
* @param      None
* @return     None
*******************************************************************************/
static void Main_InitializeTimer1n(void) {
	// Initialize and configure timer1n for ADC trigger
	HAL_SCU_MiscClockConfig(1, TIMER1n_TYPE, CLKSRC_MCLK, 2); // 48MHz/2 = 24MHz
	HAL_SCU_Timer1n_ClockConfig(T1NCLK_MCCR1);

	TIMER1n_Config.StartSync = DISABLE;
	TIMER1n_Config.ClearSync = DISABLE;
	TIMER1n_Config.ExtClock = DISABLE; // MCCR1, 24MHz
	TIMER1n_Config.MatchInterrupt = ENABLE;
	TIMER1n_Config.CaptureInterrupt = DISABLE;

	TIMER1n_Config.AData = 1000; // 1us * 10000 = 10ms

	TIMER1n_Config.PrescalerData = 24 - 1; // 24MHz / 24 = 1MHz -> 1us

	HAL_TIMER1n_Init(TIMER10, TIMER1n_PERIODIC_MODE, &TIMER1n_Config);

	NVIC_SetPriority(TIMER10_IRQn, 3);
	NVIC_EnableIRQ(TIMER10_IRQn);
}
/* --------------------------------- End Of File ------------------------------ */
