/***************************************************************************//**
* @file     flash.c
* @brief    Contains all functions support for flash on A31G22x
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     28. Sep. 2020
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/* Includes ------------------------------------------------------------------- */
#include "main_conf.h"
#include "TestCode_CFMC_ReadProtect.h"

#define SECTOR_SIZE			(512) // 512Bytes = 0x128
volatile uint32_t           passwd;


const uint8_t SubMenu[] =
"**************************************************************************\n\r"
" Code Flash - Read Protection & Password Test\n\r"
"\t - MCU: A31G22x\n\r"
"\t - Core: ARM Cortex-M0+\n\r"
"\t - Communicate via: UART1 - 38400 bps\n\r"
"\t - 1. View Read Protection Option Area \n\r"
"\t - 2. Set Read Protection: Level-1 \n\r"
"\t - 3. Set Read Protection: Level-2 \n\r"
"\t - 4. Set Read Protection with Password: Level-1 \n\r"
"\t - 5. Set Read Protection with Password: Level-2 \n\r"
"\t - 6. Entered Password '12345678' to Preset Register \n\r"
"\t - 7. Entered Password '0x9AE0DAAF' to Preset Register \n\r"
"\t - 8. Insert Password '12345678' value \n\r"
"\t - 9. Insert Password '0x9AE0DAAF' value \n\r"
"\t - a. Chip Erase + Read Protection Option Area Erase \n\r"
"**************************************************************************\n\r";

const uint8_t cmdm[] =  "RP> ";

// Read Protection Code (Level-1)
volatile const uint32_t CODE_OPTION_TABLE_LV1[] =
{
	0x55555555, // Check Code A
	0xAAAAAAAA, // Check Code B
	0xFFFFFF39, // Read Protection(LockOnly : Level 1)
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,                                     // < 0x0F010020UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010040UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010060UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010080UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100E0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010100UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010120UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010140UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010160UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010180UL1
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101E0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
};

volatile const uint32_t CODE_OPTION_TABLE_LV1PW[] =
{
	0x55555555, // Check Code A
	0xAAAAAAAA, // Check Code B
	0xFFFFFFB9, // Read Protection(Password : Level 1)
	0xDD5B0144, 0x3F0101F8, 0xDD5F0144, 0x3F0101FC, 0xFFFFFFFF,                                     // < 0x0F010020UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010040UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010060UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010080UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100E0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010100UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010120UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010140UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010160UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010180UL1
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101E0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101F8UL
	0x9AE0DAAF, //Password Preset 1-Word(4-byte)(LSB) ; 0x0F0101F8, (Data : '1', '2', '3', '4', '5', '6', '7', '8')
	0x9AE0DAAF  //Password Preset 1-Word(4-byte)(MSB) ; 0x0F0101FC, (Data : '1', '2', '3', '4', '5', '6', '7', '8')
};

volatile const uint32_t CODE_OPTION_TABLE_LV2[] =
{
	0x55555555, // Check Code A
	0xAAAAAAAA, // Check Code B
	0xFFFFFF00, // Read Protection(LockOnly : Level 2)
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,                                     // < 0x0F010020UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010040UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010060UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010080UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100E0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010100UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010120UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010140UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010160UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010180UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101E0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101E0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
};

volatile const uint32_t CODE_OPTION_TABLE_LV2PW[] =
{
	0x55555555, // Check Code A
	0xAAAAAAAA, // Check Code B
	0xFFFFFF80, // Read Protection(Password : Level 2)
	0xDD5B0144, 0x3F0101F8, 0xDD5F0144, 0x3F0101FC, 0xFFFFFFFF,                                     // < 0x0F010020UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010040UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010060UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010080UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100E0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010100UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010120UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010140UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010160UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010180UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101E0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101F8UL
	0x9AE0DAAF, //Password Preset 1-Word(4-byte)(LSB) ; 0x0F0101F8, (Data : '1', '2', '3', '4', '5', '6', '7', '8')
	0x9AE0DAAF  //Password Preset 1-Word(4-byte)(MSB) ; 0x0F0101FC, (Data : '1', '2', '3', '4', '5', '6', '7', '8')
};

void CFMC_PrintData(uint32_t addr, int size);
void CFMC_ReadProtection(void);
void CFMC_CheckReadProtectionStatus(void);
void CFMC_getChecksum(uint32_t startaddr, uint32_t size);

void CFMC_getChecksum(uint32_t startaddr, uint32_t size) {
	int i;
	uint16_t checksum;
	uint32_t sum = 0;

	for(i = 0 ; i < size; i++) {
		sum += *((volatile unsigned char *)(startaddr+i));
	}

	checksum = (sum & 0xFFFF);
    cprintf("0x%08X ~ 0x%08X Checksum: 0x%04X\r\n", startaddr, (startaddr+size), checksum);
}

void CFMC_CheckReadProtectionStatus(void) {
	uint32_t Reg32;

	// View Read Protection Statusk
	Reg32 = CFMC->RPROT;
	cprintf("\r\nCFMC->RPROT SFR = [0x%08X]\r\n", Reg32);

	if ((Reg32 & CFMC_RPROT_DBGMOD_Msk) == CFMC_RPROT_DBGMOD_Msk) {
		_DBG("Debug Operating Mode\r\n");
	} else {
		_DBG("Debug NOT Working \r\n");
	}

	if ((Reg32 & CFMC_RPROT_SRBOOT_Msk) == CFMC_RPROT_SRBOOT_Msk) {
		_DBG("SRAM Boot Mode \r\n");
	} else {
		_DBG("Normal Mode \r\n");
	}

	if ((Reg32 & CFMC_RPROT_PWMATCH_Msk) == CFMC_RPROT_PWMATCH_Msk) {
		_DBG("Matched Password!\r\n");
	} else {
		_DBG("Wrong Password or Password has not been entered. \r\n");
	}

	if ((Reg32 & CFMC_RPROT_CERSD_Msk) == CFMC_RPROT_CERSD_Msk) {
		_DBG("Chip Erased Done\r\n");
	}

	if ((Reg32 & CFMC_RPROT_RPBERSD_Msk) == CFMC_RPROT_RPBERSD_Msk) {
		_DBG("Read Protection Block Erase Done\r\n");
	}


	if ((Reg32 & CFMC_RPROT_LVL1_EN_Msk) == CFMC_RPROT_LVL1_EN_Msk) {
		_DBG("Enable Read Protection Level 1 \r\n");

		if ((Reg32 & CFMC_RPROT_LVL1_STS_Msk) == CFMC_RPROT_LVL1_STS_Msk) {
			_DBG("Read Protection Level 1 Status \r\n");
		} else {
			_DBG("Read Protection Level 1 - Normal Status\r\n");
		}
	}

	if ((Reg32 & CFMC_RPROT_LVL2_EN_Msk) == CFMC_RPROT_LVL2_EN_Msk) {
		_DBG("Enable Read Protection Level 2 \r\n");

		if ((Reg32 & CFMC_RPROT_LVL2_STS_Msk) == CFMC_RPROT_LVL2_STS_Msk) {
			_DBG("Read Protection Level 2 Status\r\n");
		} else {
			_DBG("Read Protection Level 2 - Normal Status\r\n");
		}
	}

	if ((Reg32 & CFMC_RPROT_RPROT_Msk) == 0xFF) {
		_DBG("Unprotected! \r\n");
	}
}

void CFMC_PrintData(uint32_t DataAddr, int Size)
{
	int i;
	for(i = 0 ; i < Size ; i+= sizeof(uint32_t)) {
		if ((i > 0) && (i % 32 == 0)) {
			_DBG("\r\n");
		}
		cprintf("0x%08X ", *(volatile uint32_t *)(DataAddr + i));
	}
	_DBG("\n\r\n\r");
}

void Main_CFMC_ReadProtection(void) {
	uint8_t ch;

    CFMC_CheckReadProtectionStatus();

//        _DBG(bankswap);
    _DBG((char*)__DATE__); _DBG(" "); _DBG(__TIME__); _DBG("\r\n");
    _DBG((char*)SubMenu);

    _DBG((char*)cmdm);

    
	while(1) {
        
		ch = cgetchar();
        if ((ch > 0) && (ch <= 0x7E)) {
            if (ch == '1') {
                _DBG("<Read Protection Option Area ----------> \n\r");
                CFMC_PrintData(CFMC_OPTION_READRPOT_ADDR, SECTOR_SIZE);

                _DBG("\n\r");
            }

            if (ch == '2') {
                __disable_irq();
        
                _DBG("Set Read Protection: Level-1 ----------> \n\r\n\r");
                HAL_CFMC_EraseSector(OPTION_READPROT, CFMC_OPTION_READRPOT_ADDR);
                HAL_CFMC_ProgramPage(OPTION_READPROT, CFMC_OPTION_READRPOT_ADDR , SECTOR_SIZE, (unsigned char *)CODE_OPTION_TABLE_LV1);
                __enable_irq();

                CFMC_PrintData(CFMC_OPTION_READRPOT_ADDR, SECTOR_SIZE);
                                
                break;
            }

            if (ch == '3') {
                __disable_irq();
                _DBG("Set Read Protection: Level-2 ----------> \n\r\n\r");
                HAL_CFMC_EraseSector(OPTION_READPROT, CFMC_OPTION_READRPOT_ADDR);
                HAL_CFMC_ProgramPage(OPTION_READPROT, CFMC_OPTION_READRPOT_ADDR , SECTOR_SIZE, (unsigned char *)CODE_OPTION_TABLE_LV2);
                __enable_irq();
                
                CFMC_PrintData(CFMC_OPTION_READRPOT_ADDR, SECTOR_SIZE);                
          
                break;                
            }

            if (ch == '4') {
                __disable_irq();
        
                _DBG("Set Read Protection with Password: Level-1 ----------> \n\r\n\r");
                HAL_CFMC_EraseSector(OPTION_READPROT, CFMC_OPTION_READRPOT_ADDR);
                HAL_CFMC_ProgramPage(OPTION_READPROT, CFMC_OPTION_READRPOT_ADDR , SECTOR_SIZE, (unsigned char *)CODE_OPTION_TABLE_LV1PW);
                __enable_irq();
                
                CFMC_PrintData(CFMC_OPTION_READRPOT_ADDR, SECTOR_SIZE);   
                
                break;                
                              
            }

            if (ch == '5') {
        
                _DBG("Set Read Protection with Password: Level-2 ----------> \n\r\n\r");
                HAL_CFMC_EraseSector(OPTION_READPROT, CFMC_OPTION_READRPOT_ADDR);
                HAL_CFMC_ProgramPage(OPTION_READPROT, CFMC_OPTION_READRPOT_ADDR , SECTOR_SIZE, (unsigned char *)CODE_OPTION_TABLE_LV2PW);
                __enable_irq();
                                
                CFMC_PrintData(CFMC_OPTION_READRPOT_ADDR, SECTOR_SIZE);
                
                break;                   

            }

            if (ch == '6') {
                _DBG("Entered Password '12345678' to Preset Register (12345678) ----------> \n\r\n\r");
                passwd = 12345678;
                HAL_CFMC_SetPassword(passwd);                
                
                break;
            }

            if (ch == '7') {
                _DBG("Entered Password '0x9AE0DAAF' to Preset Register (0x9AE0DAAF) ----------> \n\r\n\r");
                passwd = 0x9AE0DAAF;                
                HAL_CFMC_SetPassword(passwd);
                
                break;
            }

            if (ch == '8') {
                _DBG("Entered Password '12345678' (12345678) ----------> \n\r\n\r");
                passwd = 12345678;                
                HAL_CFMC_InsertPassword(passwd);
                
                break;
            }

            if (ch == '9') {
                _DBG("Entered Password '0x9AE0DAAF' (0x9AE0DAAF) ----------> \n\r\n\r");
                passwd = 0x9AE0DAAF;
                HAL_CFMC_InsertPassword(passwd);
                
                break;
            }

            if (ch == 'a') {
                _DBG("Chip (Mass) Erase + Read Protection Option Area Erase ----------> \n\r\n\r");
                // !!NOTE!! This is needs to system reset to download new firmware

                __disable_irq();
                HAL_CFMC_EraseChip();
                HAL_CFMC_EraseSector(OPTION_READPROT, CFMC_OPTION_READRPOT_ADDR);
                __enable_irq();
                
			    SCU->SCR = (SCU_SCR_WTIDKY_Msk | SCU_SCR_SWRST_Msk); // Internal System Reset (nRESET are not applied bootrom)
                
            }
        }
    }

}

/* --------------------------------- End Of File ------------------------------ */
