/*****************************************************************************
* @file     TestCode_CFMC_Test.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_CFMC_Test.h"

// Peripherals Drivers
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/
// to test some area, you should be set the IROM1 Start address and the Startup in 'Options for Target(Alt+F7), Target tab.'
#define TEST_START_ADDR (0x00020000UL)
#define TEST_END_ADDR   (0x00021000UL)

/*******************************************************************************
* Private Variable
*******************************************************************************/
#define BufferSize   (4096)
static unsigned char Buffer[BufferSize];

static const char TEST_MENU[] =
"************************************************\n\r"
" Code Flash Test\n\r"
"\t - MCU: A31G22x\n\r"
"\t - Core: ARM Cortex-M0+\n\r"
"\t - Communicate via: UART1 - 38400 bps\n\r"
"\t - 1. 512 Bytes Erase and Program Test \n\r"
"\t - 2. 1KB Erase and Program Test \n\r"
"\t - 3. 4KB Erase and Program Test \n\r"
"\t - 4. Code Flash OTP Area Access Test  \n\r"
"\t - 5. Change Flash Access Timing Test \n\r"
"\t - 6. Code Flash Information \n\r"
"\t - 7. Code Flash Self-Program Test \n\r"
"\t - 8. Code Flash CRC16 Test \n\r"
"\t - h. Print Menu \n\r"
"************************************************\n\r";

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static void CFMC_PrintMemory(uint32_t *StartAddr, uint32_t Size);
static void CFMC_512ErasePGMTest(void);
static void CFMC_1kErasePGMTest(void);
static void CFMC_4kErasePGMTest(void);
static void CFMC_OTPAccesTest(void);
static void CFMC_FlashAccessTimingTest(void);
static void CFMC_CodeFlashInformation(void);
static void CFMC_CodeFlashSelfPgmTest(void);
static void CFMC_CodeFlashCrc16Test(void);

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      DFMC Test Code
* @param      None
* @return     None
*******************************************************************************/
void TestCode_CFMC_Test(void) {
	char cmd;

    cputs((char *)TEST_MENU);
    cprintf(">");

	while(1) {
		cmd = cgetchar();
        if (cmd > 0 && cmd <= 0x7F) {
            if (cmd == '1') {
                cprintf("1. 512 Bytes Erase and Program Test\n\r");
                CFMC_512ErasePGMTest();
            } else if (cmd == '2') {
                cprintf("2. 1KB Erase and Program Test \n\r");
                CFMC_1kErasePGMTest();
            } else if (cmd == '3') {
                cprintf("3. 4KB Erase and Program Test \n\r");
                CFMC_4kErasePGMTest();
            } else if (cmd == '4') {
                cprintf("4. Code Flash OTP Area Access Test \n\r");
                CFMC_OTPAccesTest();
            } else if (cmd == '5') {
                cprintf("5. Change Flash Access Timing Test \n\r");
                CFMC_FlashAccessTimingTest();
            } else if (cmd == '6') {
                cprintf("6. Code Flash Information \n\r");
                CFMC_CodeFlashInformation();
            } else if (cmd == '7') {
                cprintf("7. Code Flash Self-PGM Test \n\r");
                CFMC_CodeFlashSelfPgmTest();
            } else if (cmd == '8') {
                cprintf("8. Code Flash CRC16 Test \n\r");
                CFMC_CodeFlashCrc16Test();
            } else if (cmd == 'h') {
                cprintf(TEST_MENU);
            } else if (cmd == 'x') {
                break;
            } else {
                cprintf("\n\r");
            }
        }
    } // loop
}

/*******************************************************************************
* @brief      CFMC_PrintMemory
* @param      None
* @return     None
*******************************************************************************/
static void CFMC_PrintMemory(uint32_t *StartAddr, uint32_t Size) {
	int i;

	for(i = 0 ; i < (Size/sizeof(uint32_t)) ; i++) {
		if ((i > 0) && (i % 4 == 0)) {
			cprintf("\r\n");
		}
		cprintf("0x%08X ", *(volatile uint32_t *)(StartAddr + i));
	}
	cprintf("\r\n"); cprintf("\r\n");
}
/*******************************************************************************
* @brief      CFMC_512ErasePGMTest
* @param      None
* @return     None
*******************************************************************************/
static void CFMC_512ErasePGMTest(void)
{
	int	i;
	uint32_t SectorSize = 512;
	int result;
	Bool Valid;

	cprintf("Test Area: 0x%08X ~ 0x%08X\r\n", TEST_START_ADDR, TEST_END_ADDR-1);
	// Fill init value to buffer
	for(i = 0 ; i < SectorSize ; i++) {
		Buffer[i] = 0xFF;
	}

	// CODE Area ERASE Test ///////////////////////////////////////////////////////
	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize) {
		__disable_irq();
		result = HAL_CFMC_EraseSector(0, i); // 512B Sector Erase
		__enable_irq();

		if (result) {
            cprintf("ERS Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}

	if (Valid == TRUE) {
		cprintf("ERS TEST OK\r\n");
	} else {
		cprintf("ERS TEST FAIL\r\n");
	}


	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize) {
		result = HAL_CFMC_Verify(i, SectorSize, Buffer);

		if (result != (i + SectorSize)) {
			cprintf("ERS VERIFY Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}
	if (Valid ==  TRUE) {
		cprintf("ERS VERIFY OK\r\n");
	} else {
		cprintf("ERS VERIFY FAIL\r\n");
	}

	// Fill a value to buffer
	for(i = 0 ; i < SectorSize ; i++) {
		Buffer[i] = (i & 0xff);
	}

	// CODE Area PROGRAM Test ///////////////////////////////////////////////////////
	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize) {
		__disable_irq();
		result = HAL_CFMC_ProgramPage(0, i , SectorSize, Buffer);
		__enable_irq();

		if (result) {
			cprintf("PGM Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}

	if (Valid == TRUE) {
		cprintf("PGM TEST OK\r\n");
	} else {
		cprintf("PGM TEST FAIL\r\n");
	}


	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize)
	{
		result = HAL_CFMC_Verify(i, SectorSize, Buffer);

		if (result != (i + SectorSize)) {
			cprintf("PGM VERIFY Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}

	if (Valid == TRUE) {
		cprintf("PGM VERIFY OK\r\n");
	} else {
		cprintf("PGM VERIFY FAIL\r\n");
	}

	cprintf("\r\n");

}

/*******************************************************************************
* @brief      CFMC_1kErasePGMTest
* @param      None
* @return     None
*******************************************************************************/
static void CFMC_1kErasePGMTest(void) {
	int	i;
	uint32_t SectorSize = 1024;
	int result;
	Bool Valid;

	cprintf("Test Area: 0x%08X ~ 0x%08X\r\n", TEST_START_ADDR, TEST_END_ADDR-1);
	// Fill init value to buffer
	for(i = 0 ; i < SectorSize ; i++) {
		Buffer[i] = 0xFF;
	}

	// CODE Area ERASE Test ///////////////////////////////////////////////////////
	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize) {
		__disable_irq();
		result = HAL_CFMC_EraseSector1k(0, i); // 1KB Sector Erase
		__enable_irq();

		if (result) {
			cprintf("ERS Error!  at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}

	if (Valid == TRUE) {
		cprintf("ERS TEST OK\r\n");
	} else {
		cprintf("ERS TEST FAIL\r\n");
	}


	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i += SectorSize) {
		result = HAL_CFMC_Verify(i, SectorSize, Buffer);

		if (result != (i + SectorSize)) {
			cprintf("ERS VERIFY Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}
	if (Valid ==  TRUE) {
		cprintf("ERS VERIFY OK\r\n");
	} else {
		cprintf("ERS VERIFY FAIL\r\n");
	}

	// Fill a value to buffer
	for(i = 0 ; i < SectorSize ; i++) {
		Buffer[i] = (i & 0xff);
	}

	// CODE Area PROGRAM Test ///////////////////////////////////////////////////////
	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize) {
		__disable_irq();
		result = HAL_CFMC_ProgramPage(0, i , SectorSize, Buffer);
		__enable_irq();

		if (result) {
			cprintf("PGM Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}

	if (Valid == TRUE) {
		cprintf("PGM TEST OK\r\n");
	} else {
		cprintf("PGM TEST FAIL\r\n");
	}


	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize)
	{
		result = HAL_CFMC_Verify(i, SectorSize, Buffer);

		if (result != (i + SectorSize)) {
			cprintf("PGM VERIFY Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}

	if (Valid == TRUE) {
		cprintf("PGM VERIFY OK\r\n");
	} else {
		cprintf("PGM VERIFY FAIL\r\n");
	}

	cprintf("\r\n");
}

/*******************************************************************************
* @brief      CFMC_4kErasePGMTest
* @param      None
* @return     None
*******************************************************************************/
static void CFMC_4kErasePGMTest(void) {
	int	i;
	uint32_t SectorSize = 4096;
	int result;
	Bool Valid;

	cprintf("Test Area: 0x%08X ~ 0x%08X\r\n", TEST_START_ADDR, TEST_END_ADDR-1);
	// Fill init value to buffer
	for(i = 0 ; i < SectorSize ; i++) {
		Buffer[i] = 0xFF;
	}

	// CODE Area ERASE Test ///////////////////////////////////////////////////////
	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize) {
		__disable_irq();
		result = HAL_CFMC_EraseSector4k(0, i); // 4K Sector Erase
		__enable_irq();

		if (result) {
			cprintf("ERS Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}

	if (Valid == TRUE) {
		cprintf("ERS TEST OK\r\n");
	} else {
		cprintf("ERS TEST FAIL\r\n");
	}


	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize) {
		result = HAL_CFMC_Verify(i, SectorSize, Buffer);

		if (result != (i + SectorSize)) {
			cprintf("ERS VERIFY Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}
	if (Valid ==  TRUE) {
		cprintf("ERS VERIFY OK\r\n");
	} else {
		cprintf("ERS VERIFY FAIL\r\n");
	}

	// Fill a value to buffer
	for(i = 0 ; i < SectorSize ; i++) {
		Buffer[i] = (i & 0xff);
	}

	// CODE Area PROGRAM Test ///////////////////////////////////////////////////////
	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize) {
		__disable_irq();
		result = HAL_CFMC_ProgramPage(0, i , SectorSize, Buffer);
		__enable_irq();

		if (result) {
			cprintf("PGM Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}

	if (Valid == TRUE) {
		cprintf("PGM TEST OK\r\n");
	} else {
		cprintf("PGM TEST FAIL\r\n");
	}


	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize)
	{
		result = HAL_CFMC_Verify(i, SectorSize, Buffer);

		if (result != (i + SectorSize)) {
			cprintf("PGM VERIFY Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}

	if (Valid == TRUE) {
		cprintf("PGM VERIFY OK\r\n");
	} else {
		cprintf("PGM VERIFY FAIL\r\n");
	}

	cprintf("\r\n");
}

/*******************************************************************************
* @brief      CFMC_OTPAccesTest
* @param      None
* @return     None
*******************************************************************************/
static void CFMC_OTPAccesTest(void)
{
	int	i;
	uint32_t SectorSize = 512;

	// Fill Buffer
	for(i=0; i < SectorSize ; i++) {
		Buffer[i]= i & 0xff;
	}
	__disable_irq();

	// OTP Area Test ////////////////////////////////////////////////////////
	// Erase
	HAL_CFMC_EraseSector(OPTION_USERDATA0, CFMC_OPTION_USERDATA0_ADDR);
	HAL_CFMC_EraseSector(OPTION_USERDATA1, CFMC_OPTION_USERDATA1_ADDR);
	HAL_CFMC_EraseSector(OPTION_USERDATA2, CFMC_OPTION_USERDATA2_ADDR);
	HAL_CFMC_EraseSector(OPTION_READPROT, CFMC_OPTION_READRPOT_ADDR);
	HAL_CFMC_EraseSector(OPTION_BANKSWAP, CFMC_OPTION_BANKSWAP_ADDR);

	cprintf("\r\nOTP-USERDATA0 ---------------------------------------\r\n");
	CFMC_PrintMemory((uint32_t *)CFMC_OPTION_USERDATA0_ADDR, SectorSize);
	cprintf("\r\nOTP-USERDATA1 ---------------------------------------\r\n");
	CFMC_PrintMemory((uint32_t *)CFMC_OPTION_USERDATA1_ADDR, SectorSize);
	cprintf("\r\nOTP-USERDATA2 ---------------------------------------\r\n");
	CFMC_PrintMemory((uint32_t *)CFMC_OPTION_USERDATA2_ADDR, SectorSize);
	cprintf("\r\nOTP-BANKSWAP ---------------------------------------\r\n");
	CFMC_PrintMemory((uint32_t *)CFMC_OPTION_BANKSWAP_ADDR, SectorSize);
	cprintf("\r\nOTP-READPROT ---------------------------------------\r\n");
	CFMC_PrintMemory((uint32_t *)CFMC_OPTION_READRPOT_ADDR, SectorSize);

#if 0
	// Program OTP Access
	HAL_CFMC_ProgramPage(OPTION_USERDATA0, CFMC_OPTION_USERDATA0_ADDR, SectorSize, Buffer);
	HAL_CFMC_ProgramPage(OPTION_USERDATA1, CFMC_OPTION_USERDATA1_ADDR, SectorSize, Buffer);
	HAL_CFMC_ProgramPage(OPTION_USERDATA2, CFMC_OPTION_USERDATA2_ADDR, SectorSize, Buffer);
	HAL_CFMC_ProgramPage(OPTION_BANKSWAP, CFMC_OPTION_READRPOT_ADDR, SectorSize, Buffer);
	HAL_CFMC_ProgramPage(OPTION_READPROT, CFMC_OPTION_BANKSWAP_ADDR, SectorSize, Buffer);

	cprintf("\r\nOTP-USERDATA0 ---------------------------------------\r\n");
	CFMC_PrintMemory((uint32_t *)CFMC_OPTION_USERDATA0_ADDR, SectorSize);
	cprintf("\r\nOTP-USERDATA1 ---------------------------------------\r\n");
	CFMC_PrintMemory((uint32_t *)CFMC_OPTION_USERDATA1_ADDR, SectorSize);
	cprintf("\r\nOTP-USERDATA2 ---------------------------------------\r\n");
	CFMC_PrintMemory((uint32_t *)CFMC_OPTION_USERDATA2_ADDR, SectorSize);
	cprintf("\r\nOTP-BANKSWAP ---------------------------------------\r\n");
	CFMC_PrintMemory((uint32_t *)CFMC_OPTION_BANKSWAP_ADDR, SectorSize);
	cprintf("\r\nOTP-READPROT ---------------------------------------\r\n");
	CFMC_PrintMemory((uint32_t *)CFMC_OPTION_READRPOT_ADDR, SectorSize);
	/////////////////////////////////////////////////////////////////////////
#endif
	__enable_irq();

}

/*******************************************************************************
* @brief      CFMC_FlashAccessTimingTest
* @param      None
* @return     None
*******************************************************************************/
static void CFMC_FlashAccessTimingTest(void) {
	uint32_t Reg16;
	int i;

	// Check Current Status
	cprintf("Current Flash Status ------------------------------------------\r\n");
        cprintf("Flash Access Timing: %d\r\n", HAL_CFMC_GetFlashAccessTiming());
        cprintf("System Core Clock: %d\r\n", SystemCoreClock);
	cprintf("Gets Flash Wait Time: %d\r\n\r\n", HAL_CFMC_GetFlashWaitValue());

	// Set Flash Access Timing Value
	cprintf("Test Start ------------------------------------------\r\n");
	Reg16 = CFMC_WAIT_MAX_INDEX; // Max flash wait time value that can be set
	for (i = 0; i < Reg16 ; i++) {
		HAL_CFMC_SetflashAccesstiming(SystemCoreClock, i, ENABLE); // Setting Flash Wait Time from 0 ~ 32
        cprintf("System Core Clock: %d\r\n", SystemCoreClock);
        cprintf("Flash Access Timing: %d\r\n", HAL_CFMC_GetFlashAccessTiming());

		cprintf("Sets Flash Wait Time : %d\r\n", i);
		cprintf("Gets Flash Wait Time : %d\r\n", HAL_CFMC_GetFlashWaitValue());
		if (i != HAL_CFMC_GetFlashWaitValue()) { // If setting value and current value are not matched, print the error message.
			cprintf("-->The setting value exceeds the specification!\r\n\r\n");
		}
	}
	cprintf("\r\n");
}

/*******************************************************************************
* @brief      CFMC_CodeFlashInformation
* @param      None
* @return     None
*******************************************************************************/
static void CFMC_CodeFlashInformation(void) {
	volatile uint32_t * hwid;

	// Check Flash Size
	hwid = (volatile uint32_t *)CFMC_HWID_REG_ADDR;
	cprintf("Flash Size is ");
	switch (*hwid) {
		case 0x41414000:
			cprintf("128KB\r\n");
		break;
		case 0x31313000:
			cprintf("64KB\r\n");
		break;
		case 0x31316000:
			cprintf("256KB\r\n");
		break;
		case 0x3131FF00:
			cprintf("256KB, ERROR\r\n");
		break;
		default:
			cprintf("Error!\r\n");
		break;
	}
	cprintf("\r\n");
}

/*******************************************************************************
* @brief      CFMC_CodeFlashSelfPgmTest
* @param      None
* @return     None
*******************************************************************************/
static void CFMC_CodeFlashSelfPgmTest(void) {
	#if 0
	uint32_t SectorSize = 512;
	int i;


	// Fill init value to buffer
	for(i = 0 ; i < SectorSize ; i++) {
		Buffer[i] = i;
	}

	// CODE Area SELF-ERASE Test ///////////////////////////////////////////////////////
	cprintf("SELF-ERS \r\n");
	__disable_irq();
	CFMC_SelfErase(0xFF00, 512); // 512 Sector Erase
	__enable_irq();
	CFMC_PrintMemory((uint32_t *)0xFF00, 512);

//	// CODE Area SELF-PROGRAM Test ///////////////////////////////////////////////////////
	cprintf("SELF-PGM \r\n");
	__disable_irq();
	CFMC_SelfProgram(0xFF00 , SectorSize, Buffer);
	__enable_irq();
	CFMC_PrintMemory((uint32_t *)0xFF00, 512);
	#endif


	uint32_t SectorSize = 512;
	int i;
	int result;
	Bool Valid;

		cprintf("Test Area: 0x%08X ~ 0x%08X\r\n", TEST_START_ADDR, TEST_END_ADDR-1);
	// Fill init value to buffer
	for(i = 0 ; i < SectorSize ; i++) {
		Buffer[i] = 0xFF;
	}

	// CODE Area ERASE Test ///////////////////////////////////////////////////////
	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize) {
		__disable_irq();
		result = HAL_CFMC_SelfErase(i, SectorSize); // 4K Sector Erase
		__enable_irq();

		if (result) {
			cprintf("SELF-ERS Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}

	if (Valid == TRUE) {
		cprintf("SELF-ERS TEST OK\r\n");
	} else {
		cprintf("SELF-ERS TEST FAIL\r\n");
	}


	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize) {
		result = HAL_CFMC_Verify(i, SectorSize, Buffer);

		if (result != (i + SectorSize)) {
			cprintf("SELF-ERS VERIFY Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}
	if (Valid ==  TRUE) {
		cprintf("SELF-ERS VERIFY OK\r\n");
	} else {
		cprintf("SELF-ERS VERIFY FAIL\r\n");
	}

	// Fill a value to buffer
	for(i = 0 ; i < SectorSize ; i++) {
		Buffer[i] = (i & 0xff);
	}

	// CODE Area PROGRAM Test ///////////////////////////////////////////////////////
	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize) {
		__disable_irq();
		result = HAL_CFMC_SelfProgram(i , SectorSize, Buffer);
		__enable_irq();

		if (result) {
			cprintf("SELF-PGM Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}

	if (Valid == TRUE) {
		cprintf("SELF-PGM TEST OK\r\n");
	} else {
		cprintf("SELF-PGM TEST FAIL\r\n");
	}


	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize)
	{
		result = HAL_CFMC_Verify(i, SectorSize, Buffer);

		if (result != (i + SectorSize)) {
			cprintf("SELF-PGM VERIFY Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}

	if (Valid == TRUE) {
		cprintf("SELF-PGM VERIFY OK\r\n");
	} else {
		cprintf("SELF-PGM VERIFY FAIL\r\n");
	}

	cprintf("\r\n");
}


/*******************************************************************************
* @brief      CFMC_CodeFlashCrc16Test
* @param      None
* @return     None
*******************************************************************************/
static void CFMC_CodeFlashCrc16Test(void) {
	uint16_t Crc = 0;
	int i;

	CFMC->CFG |= (1 << CFMC_CFG_CRCINIT_Pos); // and Initialize CRC16 Calc. value.
	CFMC->CFG &= ~(1 << CFMC_CFG_CRCINIT_Pos); // reset CRC16 Calc. value

	CFMC->CFG |= (1 << CFMC_CFG_CRCEN_Pos); //Enable CRC16
	__NOP(); __NOP(); __NOP();

	for(i = 0 ; i < 255; i++) {
        Crc = CFMC->CRCCCITT;
        cprintf("%03d 0x%04X\r\n", i, (Crc & 0xFFFF));
	}

	CFMC->CFG &= ~(1 << CFMC_CFG_CRCEN_Pos);	// Disable CRC-CCITT of CFMC
}

/* --------------------------------- End Of File ------------------------------ */

