/*****************************************************************************
* @file     TestCode_CORE_Systick.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     18. JUL. 2019
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_CORE_Systick.h"
#include "main_conf.h"

/* Definitions for VTOR TEST */
/* Referenced 4.3.4. Vector Table Offset Register - ARM Infocenter */
#define	VECTOR_TABLE_RELOCATION				// VTOR TEST (define : SRAM, undifine : Code Flash)

#define	SRAM_BASE               (0x20002000)	// SRAM BASE Address
#define	FLASH_BASE              (0x00000000)	// FLASH BASE Address
#define	VTOR_REG                (0xE000ED08)	// Vector Table Offset Register Address (
#define	VTOR_TBLBASE_Msk        (1 << 29)
#define	VTOR_TBLBASE_Pos        (29)
#define	VTOR_TBLOFF_Msk         (0x1FFFFF << 7)
#define	VTOR_TBLOFF_Pos         (7)

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint32_t MilliSec;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static void vectorTableOffsetTest(void);
static void NVIC_SetVectorTable(uint32_t NVIC_VectTab);

/*******************************************************************************
* Public Function
*******************************************************************************/
void SysTick_Handler_CoreTest(void) {    // SysTick Interrupt Handler @ 1000Hz
	if(MilliSec) {
        MilliSec--;
    }
}

/**********************************************************************
 * @brief		Vector Table Offset Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
static void vectorTableOffsetTest(void) {
	uint32_t Reg32;
	uint32_t VectOffs;

	Reg32 = *(volatile unsigned int *)VTOR_REG;

	if(Reg32&VTOR_TBLBASE_Msk) {
		cputs("Table base is in RAM ");
		VectOffs = SRAM_BASE | ((Reg32&VTOR_TBLOFF_Msk)>>VTOR_TBLOFF_Pos);
	} else {
		cputs("Table base is in CODE FLASH ");
		VectOffs = FLASH_BASE | ((Reg32&VTOR_TBLOFF_Msk)>>VTOR_TBLOFF_Pos);
	}
	cprintf("%08X\r\n\r\n", VectOffs);
}

/*********************************************************************//**
 * @brief		NVIC_SetVectorTable
 * @param[in]	None
 * @return 		None
 **********************************************************************/
static void NVIC_SetVectorTable(uint32_t NVIC_VectTab) {
	SCB->VTOR = NVIC_VectTab;
}

/**********************************************************************
 * @brief		Systick Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_CORE_Systick(void) {
	int	i;

#ifdef VECTOR_TABLE_RELOCATION
//	NVIC_SetVectorTable(SRAM_BASE);
	NVIC_SetVectorTable(FLASH_BASE);
#endif

	__disable_irq();

	_DBG("SysTick Start - Blink LEDs\r\n");

    for (i = 2 ; i < 8; i++) {
        // Test Pin setting PE2 ~ PE7
        HAL_GPIO_ConfigOutput(PE, i, PCU_MODE_PUSH_PULL);
        HAL_GPIO_ConfigPullup(PE, i, PCU_PUPD_DISABLE);
        HAL_GPIO_SetPin(PE, _BIT(i));
    }

#ifdef VECTOR_TABLE_RELOCATION
	vectorTableOffsetTest();
#endif

#if 0
    // Use systick based core clock
	SysTick_Config(SystemCoreClock/1000);  //   1msec interrupt  (Use Core Clock)
#else


    #if 0
    HAL_SCU_MiscClockConfig(1, SYSTICK_TYPE, CLKSRC_MCLK, 0);   // if STDIV=0, SYSTICK Interrupt clock is stopped
	SysTick_Config(SystemCoreClock/1000);  //   1msec interrupt  (Use Core Clock)
    #endif

    #if 0
    HAL_SCU_MiscClockConfig(1, SYSTICK_TYPE, CLKSRC_MCLK, 1);   // if STDIV=1, SYSTICK Interrupt clock is stopped
 	SysTick_Config(SystemCoreClock/1000);  //   1msec interrupt  (Use Core Clock)
    #endif

    #if 1
    HAL_SCU_MiscClockConfig(1, SYSTICK_TYPE, CLKSRC_MCLK, 48);   // 48MHz/48 = 1MHz PLL, 1us Systick
	SysTick_Config(1000000/1000);  //   1msec interrupt  (Use Core Clock)
    #endif

    #if 0
    HAL_SCU_HSI_ClockConfig(HSI_EN);
    HAL_SCU_MiscClockConfig(1, SYSTICK_TYPE, CLKSRC_HSI, 32);      // 32MHz/32 = 1MHz HSI, 1us Systick
    SysTick_Config(1000000/1000);  //   1msec interrupt  (Use Core Clock)
    #endif

    #if 0
    HAL_SCU_SetHSE(HSE_EN);
    HAL_SCU_MiscClockConfig(1, SYSTICK_TYPE, CLKSRC_HSE, 8);       // 8MHz/8   = 1MHz HSE, 1us Systick
    SysTick_Config(1000000/1000);  //   1msec interrupt  (Use Core Clock)
    #endif

    #if 0
    HAL_SCU_MiscClockConfig(1, SYSTICK_TYPE, CLKSRC_LSI, 1);       // 500kHz/1   = 500kHz HSE, 1us Systick
    SysTick_Config(500000/1000);  //   1msec interrupt  (Use Core Clock)
    #endif

    // Configure Systick Timer
    SysTick->CTRL &= ~SysTick_CTRL_CLKSOURCE_Msk;   // Use a external clock source selected by SCU_MCCR1
#endif

	__enable_irq();

	while(1) {
		MilliSec = 200;
		while(MilliSec);

		for(i = 2; i < 8; i++) {
			MilliSec = 50;
			while(MilliSec);
			HAL_GPIO_ClearPin(PE,_BIT(i));
		}

		for(i = 2; i < 8; i++) {
			MilliSec = 50;
			while(MilliSec);
			HAL_GPIO_SetPin(PE,_BIT(i));
		}

	}
}


/* --------------------------------- End Of File ------------------------------ */
