/*****************************************************************************
* @file     TestCode_CRC_AutoTest.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     12. JUL. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_CRC_AutoTest.h"
#include "main_conf.h"

// Macros
#define MAX_TEST_STRING				(10)
#define MAX_BUFFER_LENGTH			(10)

#define ASCII_CODE_BS				(0x08) // Back space
#define ASCII_CODE_LF				(0x0A) // '\n'
#define ASCII_CODE_CR				(0x0D) // '\r'
#define ASCII_CODE_SPACE			(0x20) // Space
#define ASCII_CODE_DEL				(0x7F) // Delete

/*******************************************************************************
* Private Typedef
*******************************************************************************/
typedef enum {
	CRC_AUTO_TEST_0,
	CRC_AUTO_TEST_1,
	CRC_AUTO_TEST_2,
	CRC_AUTO_TEST_3,
	CRC_AUTO_TEST_4,
	CRC_AUTO_TEST_5,
	CRC_AUTO_TEST_6,
	CRC_AUTO_TEST_7,
	CRC_AUTO_TEST_MAX
} CRC_AUTO_TEST_Type;

typedef struct {
	FunctionalState InverseOutput; /*! Inverse CRC Output data */
	FunctionalState ReverseOutput; /*! Reverse CRC Output data */
	CRC_FIRST_BIT_Type FirstBitInput; /*! First Bit of Input Data */
} CRC_DATA_CFG_Type;

/*******************************************************************************
* Private Variable
*******************************************************************************/
static const uint8_t TEST_MENU[] =
"************************************************\r\n"
"\t   0. CRC32 \n\r"
"\t   1. CRC16 \n\r"
"\t   2. CRC8 \n\r"
"\t   3. CRC7 (x^7 + X^3 + X^0) \n\r"
"************************************************\r\n";
static const uint8_t PROMPT_STRING[] =
"CRC> ";
static const uint8_t TEST_STRING[MAX_TEST_STRING] = {
	'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J'
};

static const CRC_DATA_CFG_Type TEST_DATA_CFG[CRC_AUTO_TEST_MAX] = {
	{ ENABLE,  ENABLE, CRC_FIRST_BIT_LSB},
	{ ENABLE,  ENABLE, CRC_FIRST_BIT_MSB},
	{ ENABLE, DISABLE, CRC_FIRST_BIT_LSB},
	{ ENABLE, DISABLE, CRC_FIRST_BIT_MSB},
	{DISABLE,  ENABLE, CRC_FIRST_BIT_LSB},
	{DISABLE,  ENABLE, CRC_FIRST_BIT_MSB},
	{DISABLE, DISABLE, CRC_FIRST_BIT_LSB},
	{DISABLE, DISABLE, CRC_FIRST_BIT_MSB}
};

static Bool ReverseInput;
static Bool ReverseOutput;
static uint32_t CRC_XOR;
static uint32_t CRC_Mask;
static uint32_t CRC_Order;
static uint32_t CRC_HighBit;
static uint32_t CRC_InitValue;
static uint32_t CRC_Polynominal;

static uint32_t CRC_Table[256];

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static uint32_t Main_ReverseData(uint32_t Data, uint32_t BitNum);
static uint32_t Main_UpdateCRCInit(uint32_t Init);
static void Main_GenerateCRCTable(void);
static uint32_t Main_CalculateByTable(const uint8_t *pData, uint32_t Length);
static uint32_t Main_CalculateByBit(const uint8_t *pData, uint32_t Length);

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      Comparator&CRC interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void CRC_IRQHandler_CrcAutoTest(void) {
	FlagStatus Status;

	Status = HAL_CRC_GetInterruptStatus();

	if (Status == SET) {
		HAL_CRC_ClearInterruptStatus();
	}
}

/*******************************************************************************
* @brief      Revesre data for CRC
* @param      Data : 32-bit data
* @param      BitNum : Bit number of 'Data'
* @return     Reversed data
*******************************************************************************/
static uint32_t Main_ReverseData(uint32_t Data, uint32_t BitNum)
{
	uint32_t i;
	uint32_t j;
	uint32_t ReverseData;

	j = 1;
	ReverseData = 0;
	for (i = 0x1UL << (BitNum - 1); i != 0; i >>= 1) {
		if (Data & i) {
			ReverseData |= j;
		}
		j <<= 1;
	}

	return (ReverseData);
}

/*******************************************************************************
* @brief      Update CRC Init value (direct alogrithm)
* @param      Init : CRC init value
* @return     New CRC init value
*******************************************************************************/
static uint32_t Main_UpdateCRCInit(uint32_t Init)
{
	uint32_t i;
	uint32_t Bit;
	uint32_t Value;

	Value = Init;
	for (i = 0; i < CRC_Order; i++) {
		Bit = Value & 1;
		if (Bit) {
			Value ^= CRC_Polynominal;
		}

		Value >>= 1;

		if (Bit) {
			Value |= CRC_HighBit;
		}
	}

	return Value;
}

/*******************************************************************************
* @brief      Generate CRC Table
* @param      None
* @return     None
*******************************************************************************/
static void Main_GenerateCRCTable(void)
{
	uint32_t i;
	uint32_t j;
	uint32_t Bit;
	uint32_t Value;

	for (i = 0; i < 256; i++) {
		Value = i;
		if (ReverseInput == TRUE) {
			Value = Main_ReverseData(Value, 8);
		}
		Value <<= (CRC_Order - 8);

		for (j = 0; j < 8; j++) {
			Bit = Value & CRC_HighBit;
			Value <<= 1;
			if (Bit) {
				Value ^= CRC_Polynominal;
			}
		}

		if (ReverseInput == TRUE) {
			Value = Main_ReverseData(Value, CRC_Order);
		}
		Value &= CRC_Mask;
		CRC_Table[i] = Value;
	}
}

/*******************************************************************************
* @brief      Calculate CRC by CRC Table
* @param      pData : pointer of Data
* @param      Length : Length of Data
* @return     Calculated value
*******************************************************************************/
static uint32_t Main_CalculateByTable(const uint8_t *pData, uint32_t Length)
{
	// suited for CRC_Polynominal orders between 8, 16, 32
	uint32_t Value;

	// direct algorithm
	Value = Main_UpdateCRCInit(CRC_InitValue);
	if (ReverseInput == TRUE) {
		Value = Main_ReverseData(Value, CRC_Order);

		while (Length--) {
			Value = ((Value >> 8) | (*pData++ << (CRC_Order - 8))) ^ CRC_Table[Value & 0xFF];
		}

		while (++Length < (CRC_Order / 8)) {
			Value = (Value >> 8) ^ CRC_Table[Value & 0xFF];
		}
	} else {
		while (Length--) {
			Value = ((Value << 8) | *pData++) ^ CRC_Table[(Value >> (CRC_Order - 8)) & 0xFF];
		}

		while (++Length < (CRC_Order / 8)) {
			Value = (Value << 8) ^ CRC_Table[ (Value >> (CRC_Order - 8))  & 0xFF];
		}
	}

	if (ReverseOutput ^ ReverseInput) {
		Value = Main_ReverseData(Value, CRC_Order);
	}

	Value ^= CRC_XOR;
	Value &= CRC_Mask;

	return Value;
}

/*******************************************************************************
* @brief      Calculate CRC by CRC bit
* @param      pData : pointer of Data
* @param      Length : Length of Data
* @return     Calculated value
*******************************************************************************/
static uint32_t Main_CalculateByBit(const uint8_t *pData, uint32_t Length)
{
	// suited for 7 crc order
	uint32_t i;
	uint32_t j;
	uint32_t Bit;
	uint32_t Data;
	uint32_t Value;

	// direct algorithm
	Value = Main_UpdateCRCInit(CRC_InitValue);
	for (i = 0; i < Length; i++) {
		Data = (uint32_t)(*pData++);
		if (ReverseInput == TRUE) {
			Data = Main_ReverseData(Data, 8);
		}

		for (j = 0x80; j != 0; j >>= 1) {
			Bit = Value & CRC_HighBit;
			Value <<= 1;
			if (Data & j) {
				Value |= 1;
			}
			if (Bit) {
				Value ^= CRC_Polynominal;
			}
		}
	}

	for (i = 0; i < CRC_Order; i++) {
		Bit = Value & CRC_HighBit;
		Value <<= 1;
		if (Bit) {
			Value ^= CRC_Polynominal;
		}
	}

	if (ReverseOutput) {
		Value = Main_ReverseData(Value, CRC_Order);
	}

	Value ^= CRC_XOR;
	Value &= CRC_Mask;

	return Value;
}

/*******************************************************************************
* @brief      TestCode_CRC_AutoTest
* @param      None
* @return     None
*******************************************************************************/
void TestCode_CRC_AutoTest(void) {
	Bool ValidCommand = FALSE;
	uint32_t i;
	uint32_t j;
	uint32_t SW_Result;
	uint32_t HW_Result;
	CRC_CFG_Type CRC_Config;
	CRC_DATA_CFG_Type *pDataConfig;
	CRC_AUTO_TEST_Type AutoTest;
	uint8_t StringBuffer[MAX_BUFFER_LENGTH];

	cputs((char *)TEST_MENU);
	cputs((char *)PROMPT_STRING);

	while (1) {

        if (ValidCommand == FALSE) {

            StringBuffer[0] = cgetchar();
            
            if ((StringBuffer[0] > 0) && (StringBuffer[0] <= 0x7F)) {
                cprintf("%s\r\n", StringBuffer[0]);
                ValidCommand = TRUE;
                switch (StringBuffer[0]) {
                    case '0' :
                        CRC_Config.Polynomial = CRC_POLY_CRC32;
                        CRC_Mask = 0xFFFFFFFFUL;
                        CRC_Order = 32;
                        CRC_HighBit = 0x80000000UL;
                        CRC_InitValue = 0xFFFFFFFFUL;
                        CRC_Polynominal = 0x04C11DB7;
                        break;

                    case '1' :
                        CRC_Config.Polynomial = CRC_POLY_CRC16;
                        CRC_Mask = 0x0000FFFFUL;
                        CRC_Order = 16;
                        CRC_HighBit = 0x00008000UL;
                        CRC_InitValue = 0;
                        CRC_Polynominal = 0x00008005UL;
                        break;

                    case '2' :
                        CRC_Config.Polynomial = CRC_POLY_CRC8;
                        CRC_Mask = 0x000000FFUL;
                        CRC_Order = 8;
                        CRC_HighBit = 0x00000080UL;
                        CRC_InitValue = 0;
                        CRC_Polynominal = 0x00000007UL;
                        break;

                    case '3' :
                        CRC_Config.Polynomial = CRC_POLY_CRC7;
                        CRC_Mask = 0x0000007FUL;
                        CRC_Order = 7;
                        CRC_HighBit = 0x00000040UL;
                        CRC_InitValue = 0;
                        CRC_Polynominal = 0x00000009UL;
                        break;

                    default:
                        ValidCommand = FALSE;
                        break;
                } /* switch */
            }
        }

        // Execute CRC calculation
		if (ValidCommand == TRUE) {
			for (i = 1; i < MAX_TEST_STRING; i++) {
				for (j = 0; j < i; j++) {
					cprintf("%c", TEST_STRING[j]);
				}
				cputc(',');

				for (AutoTest = CRC_AUTO_TEST_0; AutoTest < CRC_AUTO_TEST_MAX; AutoTest++) {
                    cprintf("%02X,", AutoTest);
					pDataConfig = (CRC_DATA_CFG_Type *)(&TEST_DATA_CFG[AutoTest]);
					CRC_Config.InverseOutput = pDataConfig->InverseOutput;
					CRC_Config.ReverseOutput = pDataConfig->ReverseOutput;
					CRC_Config.FirstBitInput = pDataConfig->FirstBitInput;
					CRC_Config.DMADoneInterrupt = DISABLE;

					if (CRC_Config.InverseOutput == ENABLE) {
						CRC_XOR = 0xFFFFFFFFUL;
					} else {
						CRC_XOR = 0;
					}

					if (CRC_Config.ReverseOutput == ENABLE) {
						ReverseOutput = TRUE;
					} else {
						ReverseOutput = FALSE;
					}

					if (CRC_Config.FirstBitInput == CRC_FIRST_BIT_LSB) {
						ReverseInput = TRUE;
					} else {
						ReverseInput = FALSE;
					}

					// Calcuate by S/W
					if (CRC_Config.Polynomial == CRC_POLY_CRC7) {
						SW_Result = Main_CalculateByBit(TEST_STRING, i);
					} else {
						Main_GenerateCRCTable();
						SW_Result = Main_CalculateByTable(TEST_STRING, i);
					}

                    cprintf("Calcuate by S/W %08X,", SW_Result);

					// Calcuate by H/W
					HAL_CRC_Init(&CRC_Config);
					HAL_CRC_SetInitVal(CRC_InitValue);
					HAL_CRC_ApplyInitVal();

					for (j = 0; j < i; j++) {
						HAL_CRC_SetInputData8(TEST_STRING[j]);
					}
					HW_Result = HAL_CRC_GetOutputData();

                    cprintf("Calcuate by H/W %08X,", HW_Result);
					if (SW_Result == HW_Result) {
						cputs("OK, ");
					} else {
						cputs("NG, ");
					}
					cputs("\r\n");
				}

                ValidCommand = FALSE;
			}
            cputs((char *)TEST_MENU);
			cputs((char *)PROMPT_STRING);
		}

	} /* while */
}

/* --------------------------------- End Of File ------------------------------ */
