/*****************************************************************************
* @file     TestCode_DAC_Interrupt2.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_DAC_Interrupt2.h"
#include "main_conf.h"


/*******************************************************************************
* Private Typedef
*******************************************************************************/
#define DMA_TEST_ADDR   (0x20004000UL)
/*******************************************************************************
* Private Variable
*******************************************************************************/
static DAC_CFG_Type DAC_Config;
static Bool DMA_Done;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* @brief      DAC interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void DAC_IRQHandler_Interrupt2(void) {
	DAC_STATUS_Type	Status;

	Status = HAL_DAC_GetStatus(DAC);		// Get DAC Flag Status

	// DAC DMA Rx Done Interrupt Flag
	if ((Status & DAC_STATUS_DMA_DONE_INT) == DAC_STATUS_DMA_DONE_INT) {
		DMA_Done = TRUE;
		HAL_DAC_ClearStatus(DAC, DAC_STATUS_DMA_DONE_INT);
	}
	// DAC DMA Underrun Interrupt Flag
	if ((Status & DAC_STATUS_DMA_UNDERRUN_INT) == DAC_STATUS_DMA_UNDERRUN_INT) {
		HAL_DAC_ClearStatus(DAC, DAC_STATUS_DMA_UNDERRUN_INT);
	}
}

/*******************************************************************************
* @brief      Main DAC Interrupt Test 2 (Memory to Peripheral)
* @param      None
* @return     None
*******************************************************************************/
void Main_DacInterruptTest2(void) {
	uint32_t DAC_Data;
	DMAC_CFG_Type DMA_Config;

	// This Project SRAM range sets 0x2000_0000 ~ 0x2000_4000
	// the test range (0x2000_4000 - 0x2000_5000) is used to test DMA operation with DAC
    
	// DAO(PA6) setting
	HAL_GPIO_ConfigOutput(PA, 6, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PA, 6, PCU_ALT_FUNCTION_3);
	HAL_GPIO_ConfigPullup(PA, 6, PCU_PUPD_DISABLE);

	// DAC setting
	DAC_Config.Stanby = DISABLE;									// DAC normal mode
	DAC_Config.DAC_Out2 = DISABLE;									// Disconnect with comparator input
	DAC_Config.DAC_Out1 = DISABLE;									// Disconnect with ADC ch.20
	DAC_Config.DAC_Out0 = ENABLE;									// Connect with DAO(PA6)
	DAC_Config.OutBuffer = ENABLE;									// Enable Output Buffer
	DAC_Config.ReloadSignal = DAC_RELOAD_SIGNAL_ALWAYS;				// DAC reload signal
	HAL_DAC_Init(DAC, &DAC_Config);										// Initialized DAC Configuration

	// DMA Configuration to use DAC Interrupt
	DMA_Config.Peripheral = DMAC_PERIPHERAL_DAC;					// Selected DMA DAC
	DMA_Config.Width = DMAC_WIDTH_HALF_WORD;						// Half-Word byte transfer (16-bit)
	DMA_Config.Direction = DMAC_DIRECTION_TX;						// Memory to Peripheral
	DMA_Config.PeripheralAddress = (uint32_t)(&(DAC->DR));			// DAC Data register
	HAL_DMAC_Init(DMAC0, &DMA_Config);								// DMAC0 Initization

	// Initialize DAC data
	DAC_Data = 0x00;
	HAL_DAC_SetData(DAC, (uint16_t)(DAC_Data << 4));					// 4-bit Left Shift to write 12-bit DAC data value

	// DAC DMA Interrupt Enable
	HAL_DAC_DmaInterrupt(DAC, DAC_DMA_UNDERRUN_INTR_DISABLE, DAC_DMA_RX_INTR_ENABLE);
	NVIC_SetPriority(DAC_IRQn, 3);
	NVIC_EnableIRQ(DAC_IRQn);

	HAL_DAC_Start(DAC);													// Start DAC Operation

	__enable_irq();

	cputs("DAC->DR values are wrote at DMA memory address\r\n");

	// Clear SRAM area
    *(volatile uint32_t *)DMA_TEST_ADDR = 0x00000000;

	// DMA Operation
	while(1) {
		DAC_Data++;															// Increased DAC value
		DAC_Data &= 0x0FFFUL;												// Masked DAC value

		DMA_Done = FALSE;
		HAL_DMAC_SetTransferCount(DMAC0, 1);								// a 16-bit data is transfered
		HAL_DMAC_SetMemoryAddress(DMAC0, 0x20004000UL);						// Memory Address (0x2000_4000)

		*(volatile uint32_t *)0x20004000 = (DAC_Data << 4);					// Write the value to memory

		HAL_DMAC_Start(DMAC0);												// DMA Ch.0 Operation Start

		while (DMA_Done == FALSE);											// Check DMA Flag
		HAL_DMAC_Stop(DMAC0);												// If DMA is done, Stop DMA operation

		cprintf("0x20004000 : 0x%08X, ", (*(volatile uint32_t *)0x20004000 >> 4));
		cprintf("DAC->DR : 0x%08X\r\n", HAL_DAC_GetData(DAC));

		if(DAC_Data >= 0xFFF) {
			DAC_Data = 0;
		}
	}
}

/* --------------------------------- End Of File ------------------------------ */
