/*****************************************************************************
* @file     TestCode_I2C_EEPROM_Polling.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     14. JUN. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_I2C_EEPROM_Polling.h"
#include "main_conf.h"

/** Select Test I2C Channel before project compile (Must be set a channel) */
//#define TEST_I2Cn	(0) // I2C0
//#define TEST_I2Cn	(1) // I2C1
#define TEST_I2Cn	(2) // I2C2

/** Max buffer length */
#define BUFFER_SIZE	(64-1) // Data Buffer Size (Excluded slave address byte size)

/** Max I2C Channel Number */
#define I2C_IP_INDEX_MAX			(3)


/** Slave Address Setting */
#define ADDR_24LC02		            (0xA2)
#define SLAVE_ADDR	                (ADDR_24LC02 >> 1) // Slave Address

/** I2C Speed */
//#define I2C_SPEED	                (100000)	// 100kHz
#define I2C_SPEED	                (400000)	// 400kHz


/** INTERVAL TEST */
// #define INTERVAL_TEST

/** SCL Low Time-out Function */
//#define SCL_TIMEOUT

/*******************************************************************************
* Private Typedef
*******************************************************************************/
// I2C_PORT_Type Structure
typedef struct {
	// Base Address
	I2C_Type *pI2Cx;
 	// SCL
	PORT_Type *pSclPort;
	uint32_t SclPort_Number;
	PCU_ALT_FUNCTION_Type SclAltFunc;
	// SDA
	PORT_Type *pSdaPort;
	uint32_t SdaPort_Number;
	PCU_ALT_FUNCTION_Type SdaAltFunc;
	// Interrupt Handler
	IRQn_Type IRQ_Type;
	// Module Name
	uint8_t I2cName[5];
} I2Cn_PORT_Type;

typedef enum {
    MASTER = 0,
    SLAVE = 1
} I2Cn_Type;

/*******************************************************************************
* Private Variable
*******************************************************************************/
static const I2Cn_PORT_Type I2C_PORT_CFG[I2C_IP_INDEX_MAX] = {
//	{I2C0, (PORT_Type *)PF,  6, PCU_ALT_FUNCTION_2, (PORT_Type *)PF, 7, PCU_ALT_FUNCTION_2, I2C0_IRQn,       "I2C0"},
	{I2C0,              PD,  0, PCU_ALT_FUNCTION_1,              PD, 1, PCU_ALT_FUNCTION_1, I2C0_IRQn,       "I2C0"},
	{I2C1,              PA,  1, PCU_ALT_FUNCTION_1,              PA, 0, PCU_ALT_FUNCTION_1, I2C1_IRQn,       "I2C1"},
//	{I2C1, (PORT_Type *)PF,  0, PCU_ALT_FUNCTION_1, (PORT_Type *)PF, 1, PCU_ALT_FUNCTION_1, I2C1_IRQn,       "I2C1"},
	{I2C2,              PC,  6, PCU_ALT_FUNCTION_1,              PC, 5, PCU_ALT_FUNCTION_1, I2C2_SPI20_IRQn, "I2C2"},
};
static const I2Cn_PORT_Type * Master = &(I2C_PORT_CFG[TEST_I2Cn]);

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static void Buffer_Init(uint8_t* buffer, uint8_t type, uint16_t len);

/*******************************************************************************
* Public Function
*******************************************************************************/
/**********************************************************************
 * @brief		I2C channel configuration
 * @param[in]	pConfig
 *                  - I2C base address
 * @param[in]	Speed
 *                  - I2C communication speed
 * @param[in]	Type
 *                  - MASTER (0)
 *                  - SLAVE  (1)
 * @return 		None
 **********************************************************************/
static void I2C_ChannelConfig(const I2Cn_PORT_Type * pConfig, uint32_t Speed, I2Cn_Type Type) {
    /* MASTER CHANNEL INIT */
	// SCL port
	HAL_GPIO_ConfigOutput(pConfig->pSclPort, pConfig->SclPort_Number, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(pConfig->pSclPort, pConfig->SclPort_Number, pConfig->SclAltFunc);
//	HAL_PCU_ConfigureDebounce(pConfig->pSclPort, pConfig->SclPort_Number, ENABLE);	// debounce enable (for Chip Test)

	// SDA port
	HAL_GPIO_ConfigOutput(pConfig->pSdaPort, pConfig->SdaPort_Number, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(pConfig->pSdaPort, pConfig->SdaPort_Number, pConfig->SdaAltFunc);
//	HAL_PCU_ConfigureDebounce(pConfig->pSdaPort, pConfig->SdaPort_Number, ENABLE);	// debounce enable (for Chip Test)

#ifdef INTERVAL_TEST
    // Interval Test
    pConfig->pI2Cx->CR |= ((2 & I2C_CR_INTERVAL_Msk) << I2C_CR_INTERVAL_Pos);
#endif

#ifdef SCL_TIMEOUT
	I2C_SetSclLowTimeout(pConfig->pI2Cx, ENABLE, ENABLE, 0xFFFFFF); // SCL Low Time Period : (1/PCLK)*4*(SLTPDR+1) = (1/32MHz)*4*(0xFFFFFF+1)2.09s
#endif

	/* Set Own Slave Address for I2C device */
    if (Type == SLAVE) {
        HAL_I2C_SetOwnSlaveAddr0(pConfig->pI2Cx, SLAVE_ADDR, DISABLE);
        HAL_I2C_SetOwnSlaveAddr1(pConfig->pI2Cx, SLAVE_ADDR, DISABLE);
    }

	// Initialize Master I2C peripheral
	HAL_I2C_Init(pConfig->pI2Cx, Speed);
}

/**********************************************************************
 * @brief		I2C and EEPROM Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Main_TestI2CEepromPolling(void) {

    int i;
    int isMatched = 0;
	uint8_t WriteBuffer[BUFFER_SIZE];
	uint8_t WriteReqBuffer[BUFFER_SIZE];
    uint8_t ReadBuffer[BUFFER_SIZE];

	I2C_M_SETUP_Type MasterCfg;

    cprintf("%s Initialization\r\n", Master->I2cName);

    /* MASTER CHANNEL INIT */
    I2C_ChannelConfig(Master, I2C_SPEED, MASTER);     // Slave Channel Setting

    cprintf("Transmission Buffer Size : %d\r\n", BUFFER_SIZE);

    /* EEPROM WRITE : Ready Writebuffer to transmit data by master */
    //-----------------------------------------------------------------------------------------------
    //| Slave Address | MSB Address | LSB Address | Data [0] | Data [1] | Data [2] | ... | Data [n] |
    //-----------------------------------------------------------------------------------------------
    Buffer_Init(WriteBuffer, 1, BUFFER_SIZE);
    MasterCfg.sl_addr7bit = SLAVE_ADDR; // Slave address
    WriteBuffer[0] = 0x00; // MSB address
    WriteBuffer[1] = 0x01; // LSB address
    MasterCfg.tx_data = WriteBuffer;
    MasterCfg.tx_length = BUFFER_SIZE;

  	cputs("\r\n EEPROM Write Test START\r\n");
    /* I2C transmission */
    HAL_I2C_MasterTransmitData(Master->pI2Cx, &MasterCfg, I2C_TRANSFER_POLLING);
 	cputs("\r\n EEPROM Write Test OK \r\n");


    /* EEPROM READ REQUEST : Master send request */
    //---------------------------------------------
    //| Slave Address | MSB Address | LSB Address |
    //---------------------------------------------
    Buffer_Init(WriteReqBuffer, 0, 2);
    MasterCfg.sl_addr7bit = SLAVE_ADDR; // Slave address
    WriteReqBuffer[0] = 0x00; // MSB address
    WriteReqBuffer[1] = 0x01; // LSB address
    MasterCfg.tx_data = WriteReqBuffer;
    MasterCfg.tx_length = 2;

    /* I2C transmission */
    HAL_I2C_MasterTransmitData(Master->pI2Cx, &MasterCfg, I2C_TRANSFER_POLLING);


    // EEPROM RECEIVE: the data is transmitted by slave
    //---------------------------------------------
    //| Slave Address | MSB Address | LSB Address |
    //---------------------------------------------
    Buffer_Init(ReadBuffer, 0, BUFFER_SIZE);
    MasterCfg.sl_addr7bit = SLAVE_ADDR;
    MasterCfg.rx_data = ReadBuffer;
    MasterCfg.rx_length = BUFFER_SIZE-2;

	cputs("\r\n EEPROM 24CXX Read Test \r\n");
    /* I2C transmission */
    HAL_I2C_MasterReceiveData(Master->pI2Cx, &MasterCfg, I2C_TRANSFER_POLLING);

    // Compare Write and Read Values
    for(i = 0; i < (BUFFER_SIZE-2); i++) {

        cprintf("W[%d]: 0x%02X ", (i+2), WriteBuffer[i+2]);
        cprintf("R[%d]: 0x%02X ", (i), ReadBuffer[i]);

        if(WriteBuffer[i+2] == ReadBuffer[i]) {
            isMatched++;
            cputs(" --> OK \r\n");
        } else {
            cputs(" --> FAIL \r\n");
        }
    }

    if (isMatched == (BUFFER_SIZE-2))
        cputs("\r\n EEPROM Read Test OK\r\n");
    else
        cputs("\r\n EEPROM Read Test False\r\n");

    /* Infinite loop */
	while(1) {}
}

/*********************************************************************//**
 * @brief		Initialize buffer
 * @param[in]	buffer	buffer to initialize
 * @param[in]	type:
 * 				- 0: Initialize buffer with 0
 * 				- 1: Initialize buffer with increment value from 0
 * @return 		None
 **********************************************************************/
static void Buffer_Init(uint8_t* buffer, uint8_t type, uint16_t len) {
	uint8_t i;

	if (type) {
		for (i = 0; i < len; i++) {
			buffer[i] = i;
		}
	}
	else {
		for (i = 0; i < len; i++) {
			buffer[i] = 0;
		}
	}
}

/* --------------------------------- End Of File ------------------------------ */
