/*****************************************************************************
* @file     TestCode_I2C_Master_Polling.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     14. JUN. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_I2C_Master_Polling.h"
#include "main_conf.h"

/** Select Test I2C Channel before project compile (Must be set a channel) */
//#define TEST_I2Cn	(0) // I2C0
//#define TEST_I2Cn	(1) // I2C1
#define TEST_I2Cn	(2) // I2C2

/** Max buffer length */
#define BUFFER_SIZE	(64-1) // Data Buffer Size (Excluded slave address byte size)

/** Max I2C Channel Number */
#define I2C_IP_INDEX_MAX			(3)


/** Slave Address Setting */
/** Master and Slave Address */
#define MASTER_ADDR	                (0xC0 >> 1) // Master Address
#define SLAVE_ADDR	                (0xA0 >> 1) // Slave Address

/** I2C Speed */
//#define I2C_SPEED	                (100000)	// 100kHz
#define I2C_SPEED	                (400000)	// 400kHz


/** INTERVAL TEST */
// #define INTERVAL_TEST

/** SCL Low Time-out Function */
//#define SCL_TIMEOUT

/*******************************************************************************
* Private Typedef
*******************************************************************************/
// I2C_PORT_Type Structure
typedef struct {
	// Base Address
	I2C_Type *pI2Cx;
 	// SCL
	PORT_Type *pSclPort;
	uint32_t SclPort_Number;
	PCU_ALT_FUNCTION_Type SclAltFunc;
	// SDA
	PORT_Type *pSdaPort;
	uint32_t SdaPort_Number;
	PCU_ALT_FUNCTION_Type SdaAltFunc;
	// Interrupt Handler
	IRQn_Type IRQ_Type;
	// Module Name
	uint8_t I2cName[5];
} I2Cn_PORT_Type;

typedef enum {
    MASTER = 0,
    SLAVE = 1
} I2Cn_Type;

/*******************************************************************************
* Private Variable
*******************************************************************************/
static const I2Cn_PORT_Type I2C_PORT_CFG[I2C_IP_INDEX_MAX] = {
	{I2C0, (PORT_Type *)PF,  6, PCU_ALT_FUNCTION_2, (PORT_Type *)PF, 7, PCU_ALT_FUNCTION_2, I2C0_IRQn,       "I2C0"},
//	{I2C0,              PD,  0, PCU_ALT_FUNCTION_1,              PD, 1, PCU_ALT_FUNCTION_1, I2C0_IRQn,       "I2C0"},
	{I2C1,              PA,  1, PCU_ALT_FUNCTION_1,              PA, 0, PCU_ALT_FUNCTION_1, I2C1_IRQn,       "I2C1"},
//	{I2C1, (PORT_Type *)PF,  0, PCU_ALT_FUNCTION_1, (PORT_Type *)PF, 1, PCU_ALT_FUNCTION_1, I2C1_IRQn,       "I2C1"},
	{I2C2,              PC,  6, PCU_ALT_FUNCTION_1,              PC, 5, PCU_ALT_FUNCTION_1, I2C2_SPI20_IRQn, "I2C2"},
};
static const I2Cn_PORT_Type * Master = &(I2C_PORT_CFG[TEST_I2Cn]);
static const I2Cn_PORT_Type * Slave = &(I2C_PORT_CFG[TEST_I2Cn]);

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static void Buffer_Init(uint8_t* buffer, uint8_t type, uint16_t len);
static void I2C_ChannelConfig(const I2Cn_PORT_Type * pConfig, uint32_t Speed, I2Cn_Type Type);

/*******************************************************************************
* Public Function
*******************************************************************************/
/**********************************************************************
 * @brief		I2C channel configuration
 * @param[in]	pConfig
 *                  - I2C base address
 * @param[in]	Speed
 *                  - I2C communication speed
 * @param[in]	Type
 *                  - MASTER (0)
 *                  - SLAVE  (1)
 * @return 		None
 **********************************************************************/
static void I2C_ChannelConfig(const I2Cn_PORT_Type * pConfig, uint32_t Speed, I2Cn_Type Type) {
    /* MASTER CHANNEL INIT */
	// SCL port
	HAL_GPIO_ConfigOutput(pConfig->pSclPort, pConfig->SclPort_Number, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(pConfig->pSclPort, pConfig->SclPort_Number, pConfig->SclAltFunc);
//	HAL_PCU_ConfigureDebounce(pConfig->pSclPort, pConfig->SclPort_Number, ENABLE);	// debounce enable (for Chip Test)

	// SDA port
	HAL_GPIO_ConfigOutput(pConfig->pSdaPort, pConfig->SdaPort_Number, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(pConfig->pSdaPort, pConfig->SdaPort_Number, pConfig->SdaAltFunc);
//	HAL_PCU_ConfigureDebounce(pConfig->pSdaPort, pConfig->SdaPort_Number, ENABLE);	// debounce enable (for Chip Test)

#ifdef INTERVAL_TEST
    // Interval Test
    pConfig->pI2Cx->CR |= ((2 & I2C_CR_INTERVAL_Msk) << I2C_CR_INTERVAL_Pos);
#endif

#ifdef SCL_TIMEOUT
	HAL_I2C_SetSclLowTimeout(pConfig->pI2Cx, ENABLE, ENABLE, 0xFFFFFF); // SCL Low Time Period : (1/PCLK)*4*(SLTPDR+1) = (1/32MHz)*4*(0xFFFFFF+1)2.09s
#endif

	/* Set Own Slave Address for I2C device */
    if (Type == SLAVE) {
        HAL_I2C_SetOwnSlaveAddr0(pConfig->pI2Cx, SLAVE_ADDR, DISABLE);
        HAL_I2C_SetOwnSlaveAddr1(pConfig->pI2Cx, SLAVE_ADDR, DISABLE);
    }

	// Initialize Master I2C peripheral
	HAL_I2C_Init(pConfig->pI2Cx, Speed);
}

/**********************************************************************
 * @brief		Main_TestI2CMasterPolling
 * @param[in]	None
 * @return 		None
 **********************************************************************/
#if 0
void Main_TestI2CMasterPolling(void) {

	uint8_t MasterWriteBuffer[BUFFER_SIZE];
    uint8_t MasterReadBuffer[BUFFER_SIZE];

	I2C_M_SETUP_Type MasterCfg;

    cprintf("%s Initialization\r\n", Master->I2cName);

    /* MASTER CHANNEL INIT */
    I2C_ChannelConfig(Master, I2C_SPEED, MASTER);   // Master Channel Setting

	__enable_irq();

    cprintf("Buffer Size : %d\r\n", BUFFER_SIZE);

    /* MASTER WRITE : Ready Writebuffer to transmit data by master */
    //--------------------------------------------------------
    //| Slave Address | Data [0] | Data [1] | ... | Data [n] |
    //--------------------------------------------------------
    Buffer_Init(MasterWriteBuffer, 0, BUFFER_SIZE); // Buffer Init
    MasterCfg.sl_addr7bit = MASTER_ADDR; // Slave address
    MasterWriteBuffer[0] = 0xAB; // Data 0
    MasterWriteBuffer[1] = 0xCD; // Data 1
    MasterWriteBuffer[2] = 0xEF; // Data 2
    MasterWriteBuffer[3] = 0x12; // Data 3
    MasterCfg.tx_data = MasterWriteBuffer;
    MasterCfg.tx_length =  4;

  	cputs("\r\n MASTER Write Test START\r\n");
     HAL_I2C_MasterTransmitData(Master->pI2Cx, &MasterCfg, I2C_TRANSFER_POLLING);
 	cputs("\r\n MASTER Write Test OK \r\n");


    /* MASTER READ: the data is transmitted by slave */
    //--------------------------------------------------------
    //| Slave Address | Data [0] | Data [1] | ... | Data [n] |
    //--------------------------------------------------------
    Buffer_Init(MasterReadBuffer, 0, BUFFER_SIZE);
    MasterCfg.sl_addr7bit = SLAVE_ADDR;
    MasterCfg.tx_data = 0x00;
    MasterCfg.tx_length = 1;
    MasterCfg.rx_data = MasterReadBuffer;
    MasterCfg.rx_length = 4;

    cputs("\r\n MASTER Read Test START\r\n");
    HAL_I2C_MasterReceiveData(Master->pI2Cx, &MasterCfg, I2C_TRANSFER_POLLING);
    cputs("\r\n MASTER Read Test OK \r\n");


    /* Infinite loop */
	while(1) {}
}

#else
void Main_TestI2CMasterPolling(void) {

    uint8_t MasterWriteBuffer[BUFFER_SIZE];
    uint8_t MasterReadBuffer[BUFFER_SIZE];

    I2C_M_SETUP_Type MasterCfg;

//    uint8_t SlaveWriteBuffer[BUFFER_SIZE];
    uint8_t SlaveReadBuffer[BUFFER_SIZE];

	I2C_S_SETUP_Type SlaveCfg;

    cprintf("%s Initialization\r\n", Master->I2cName);

    /* MASTER CHANNEL INIT */
    I2C_ChannelConfig(Master, I2C_SPEED, MASTER);   // Master Channel Setting
    I2C_ChannelConfig(Slave, I2C_SPEED, SLAVE);     // Slave Channel Setting

	__enable_irq();

    cprintf("Buffer Size : %d\r\n", BUFFER_SIZE);

    /* SLAVE READ : Read Buffer and save SlaveReadBuffer */
    //-----------------------------------------------------------------------------------------------
    //| Slave Address | Data [0] | Data [1] | ... | Data [n] |
    //-----------------------------------------------------------------------------------------------
    Buffer_Init(SlaveReadBuffer, 0, BUFFER_SIZE);
    SlaveCfg.rx_data = SlaveReadBuffer;
    SlaveCfg.rx_length =  4;

    /* MASTER WRITE : Ready Writebuffer to transmit data by master */
    //--------------------------------------------------------
    //| Slave Address | Data [0] | Data [1] | ... | Data [n] |
    //--------------------------------------------------------
    Buffer_Init(MasterWriteBuffer, 0, BUFFER_SIZE); // Buffer Init
    MasterCfg.sl_addr7bit = MASTER_ADDR; // Slave address
    MasterWriteBuffer[0] = 0xA0; // Data 0
    MasterWriteBuffer[1] = 0xAB; // Data 1
    MasterWriteBuffer[2] = 0xCD; // Data 2
    MasterWriteBuffer[3] = 0xEF; // Data 3
    MasterCfg.tx_data = MasterWriteBuffer;
    MasterCfg.tx_length =  4;

    cputs("\r\n MASTER Write Test START\r\n");
    HAL_I2C_MasterTransmitData(Master->pI2Cx, &MasterCfg, I2C_TRANSFER_POLLING);
    cputs("\r\n MASTER Write Test OK \r\n");


    cputs("\r\n Slave Read Test START\r\n");
    HAL_I2C_SlaveReceiveData(Slave->pI2Cx, &SlaveCfg, I2C_TRANSFER_POLLING);
    cputs("\r\n Slave Read Test OK \r\n");


    /* MASTER READ: the data is transmitted by slave */
    //--------------------------------------------------------
    //| Slave Address | Data [0] | Data [1] | ... | Data [n] |
    //--------------------------------------------------------
    Buffer_Init(MasterReadBuffer, 0, BUFFER_SIZE);
    MasterCfg.sl_addr7bit = SLAVE_ADDR;
    MasterCfg.tx_data = 0x00;
    MasterCfg.tx_length = 1;
    MasterCfg.rx_data = MasterReadBuffer;
    MasterCfg.rx_length = 4;

    cputs("\r\n MASTER Read Test START\r\n");
    HAL_I2C_MasterReceiveData(Master->pI2Cx, &MasterCfg, I2C_TRANSFER_POLLING);
    cputs("\r\n MASTER Read Test OK \r\n");

    /* Infinite loop */
	while(1) {}
}
#endif

/*********************************************************************//**
 * @brief		Initialize buffer
 * @param[in]	buffer	buffer to initialize
 * @param[in]	type:
 * 				- 0: Initialize buffer with 0
 * 				- 1: Initialize buffer with increment value from 0
 * @return 		None
 **********************************************************************/
static void Buffer_Init(uint8_t* buffer, uint8_t type, uint16_t len) {
	uint8_t i;

	if (type) {
		for (i = 0; i < len; i++) {
			buffer[i] = i;
		}
	}
	else {
		for (i = 0; i < len; i++) {
			buffer[i] = 0;
		}
	}
}

/* --------------------------------- End Of File ------------------------------ */
