/*****************************************************************************
* @file     TestCode_I2C_Slave_Interrupt.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     14. JUN. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_I2C_Slave_Interrupt.h"
#include "main_conf.h"

/** Select Test I2C Channel before project compile (Must be set a channel) */
//#define TEST_I2Cn	(0) // I2C0
//#define TEST_I2Cn	(1) // I2C1
#define TEST_I2Cn	(2) // I2C2

/** Max buffer length */
#define BUFFER_SIZE	(64-1) // Data Buffer Size (Excluded slave address byte size)

/** Max I2C Channel Number */
#define I2C_IP_INDEX_MAX			(3)


/** Slave Address Setting */
/** Master and Slave Address */
#define MASTER_ADDR	                (0xA0 >> 1) // Master Address
#define SLAVE_ADDR	                (0xA0 >> 1) // Slave Address

/** I2C Speed */
//#define I2C_SPEED	                (100000)	// 100kHz
#define I2C_SPEED	                (400000)	// 400kHz

/** Manual Bus Control Test */
#define I2C_MANUAL_BUS_CONTROL

/*******************************************************************************
* Private Typedef
*******************************************************************************/
// I2C_PORT_Type Structure
typedef struct {
	// Base Address
	I2C_Type *pI2Cx;
 	// SCL
	PORT_Type *pSclPort;
	uint32_t SclPort_Number;
	PCU_ALT_FUNCTION_Type SclAltFunc;
	// SDA
	PORT_Type *pSdaPort;
	uint32_t SdaPort_Number;
	PCU_ALT_FUNCTION_Type SdaAltFunc;
	// Interrupt Handler
	IRQn_Type IRQ_Type;
	// Module Name
	uint8_t I2cName[5];
} I2Cn_PORT_Type;

typedef enum {
    MASTER = 0,
    SLAVE = 1
} I2Cn_Type;

/*******************************************************************************
* Private Variable
*******************************************************************************/
static const I2Cn_PORT_Type I2C_PORT_CFG[I2C_IP_INDEX_MAX] = {
	{I2C0, (PORT_Type *)PF,  6, PCU_ALT_FUNCTION_2, (PORT_Type *)PF, 7, PCU_ALT_FUNCTION_2, I2C0_IRQn,       "I2C0"},
//	{I2C0,              PD,  0, PCU_ALT_FUNCTION_1,              PD, 1, PCU_ALT_FUNCTION_1, I2C0_IRQn,       "I2C0"},
	{I2C1,              PA,  1, PCU_ALT_FUNCTION_1,              PA, 0, PCU_ALT_FUNCTION_1, I2C1_IRQn,       "I2C1"},
//	{I2C1, (PORT_Type *)PF,  0, PCU_ALT_FUNCTION_1, (PORT_Type *)PF, 1, PCU_ALT_FUNCTION_1, I2C1_IRQn,       "I2C1"},
	{I2C2,              PC,  6, PCU_ALT_FUNCTION_1,              PC, 5, PCU_ALT_FUNCTION_1, I2C2_SPI20_IRQn, "I2C2"},
};

static const I2Cn_PORT_Type * Slave = &(I2C_PORT_CFG[TEST_I2Cn]);

static Bool complete;  // Interrupt Flag
static Bool isMasterMode = TRUE; // Interrupt Master FLag

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static void Buffer_Init(uint8_t* buffer, uint8_t type, uint16_t len);
static void I2C_ChannelConfig(const I2Cn_PORT_Type * pConfig, uint32_t Speed, I2Cn_Type Type);

/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* @brief      I2C IRQ Function
* @param      None
* @return     None
*******************************************************************************/
static void Main_I2CnIRQHandler(I2C_Type *pI2Cx) {
#ifdef SCL_TIMEOUT
	uint32_t Reg32;

	Reg32 = pI2Cx->ST;
	if ((Reg32 & I2C_ST_SLT_FLAG) == I2C_ST_SLT_FLAG) {
		HAL_I2C_ClearSclLowTimeoutFlag(pI2Cx); // Clear SCL Low Time-out
	}
#endif
	if (isMasterMode) { // Master Mode
		HAL_I2C_Interrupt_MasterHandler(pI2Cx);
		if (HAL_I2C_MasterTransferComplete(pI2Cx)) {
			complete = TRUE;
		}
	} else { // Slave Mode
		HAL_I2C_Interrupt_SlaveHandler(pI2Cx);
		if (HAL_I2C_SlaveTransferComplete(pI2Cx)) {
			complete = TRUE;
		}
	}
}
/*******************************************************************************
* @brief      I2C0 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void I2C0_IRQHandler_SlaveInterrupt(void) {
	Main_I2CnIRQHandler(I2C0);
}

/*******************************************************************************
* @brief      I2C1 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void I2C1_IRQHandler_SlaveInterrupt(void) {
	Main_I2CnIRQHandler(I2C1);
}

/*******************************************************************************
* @brief      I2C2 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void I2C2_SPI20_IRQHandler_SlaveInterrupt(void) {
	Main_I2CnIRQHandler(I2C2);
}

static void I2C_ChannelConfig(const I2Cn_PORT_Type * pConfig, uint32_t Speed, I2Cn_Type Type) {
    /* MASTER CHANNEL INIT */
	// SCL port
	HAL_GPIO_ConfigOutput(pConfig->pSclPort, pConfig->SclPort_Number, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(pConfig->pSclPort, pConfig->SclPort_Number, pConfig->SclAltFunc);
//	HAL_PCU_ConfigureDebounce(pConfig->pSclPort, pConfig->SclPort_Number, ENABLE);	// debounce enable (for Chip Test)

	// SDA port
	HAL_GPIO_ConfigOutput(pConfig->pSdaPort, pConfig->SdaPort_Number, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(pConfig->pSdaPort, pConfig->SdaPort_Number, pConfig->SdaAltFunc);
//	HAL_PCU_ConfigureDebounce(pConfig->pSdaPort, pConfig->SdaPort_Number, ENABLE);	// debounce enable (for Chip Test)

	/* Set Own Slave Address for I2C device */
    if (Type == SLAVE) {
        HAL_I2C_SetOwnSlaveAddr0(Slave->pI2Cx, SLAVE_ADDR, DISABLE);
        HAL_I2C_SetOwnSlaveAddr1(Slave->pI2Cx, SLAVE_ADDR, DISABLE);
    }

	// Initialize Master I2C peripheral
	HAL_I2C_Init(pConfig->pI2Cx, Speed);
}

/**********************************************************************
 * @brief		I2C and EEPROM Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Main_TestI2CSlaveInterrupt(void) {

	uint8_t SlaveWriteBuffer[BUFFER_SIZE];
    uint8_t SlaveReadBuffer[BUFFER_SIZE];
	I2C_S_SETUP_Type SlaveCfg;

    cprintf("%s Initialization\r\n", Slave->I2cName);

    /* MASTER CHANNEL INIT */
    I2C_ChannelConfig(Slave, I2C_SPEED, SLAVE);     // Slave Channel Setting

#ifdef I2C_MANUAL_BUS_CONTROL
    HAL_I2C_SetManualBus(Slave->pI2Cx, I2C_SCL, I2C_OUTPUT_LOW, ENABLE);
    HAL_I2C_SetManualBus(Slave->pI2Cx, I2C_SCL, I2C_OUTPUT_HIGH_OPENDRAIN, ENABLE);
    HAL_I2C_SetManualBus(Slave->pI2Cx, I2C_SDA, I2C_OUTPUT_LOW, ENABLE);
    HAL_I2C_SetManualBus(Slave->pI2Cx, I2C_SDA, I2C_OUTPUT_HIGH_OPENDRAIN, ENABLE);
#endif

	__enable_irq();

    cprintf("Buffer Size : %d\r\n", BUFFER_SIZE);

    /* SLAVE READ : Read Buffer and save SlaveReadBuffer */
    //-----------------------------------------------------------------------------------------------
    //| Slave Address | Data [0] | Data [1] | ... | Data [n] |
    //-----------------------------------------------------------------------------------------------
	Buffer_Init(SlaveReadBuffer, 0, BUFFER_SIZE);
    SlaveCfg.rx_data = SlaveReadBuffer;
    SlaveCfg.rx_length =  4;

 	cputs("\r\n Slave Read Test START\r\n");
    HAL_I2C_SlaveReceiveData(Slave->pI2Cx, &SlaveCfg, I2C_TRANSFER_INTERRUPT);
    while(!complete);
    __NOP();
 	cputs("\r\n Slave Read Test OK \r\n");


    /* SLAVE WRITE : Writes Buffer and Transmits to master */
    //-----------------------------------------------------------------------------------------------
    //| Slave Address | Data [0] | Data [1] | ... | Data [n] |
    //-----------------------------------------------------------------------------------------------
	/* Initialize buffer */
	Buffer_Init(SlaveWriteBuffer, 0, BUFFER_SIZE);

    SlaveWriteBuffer[0]=MASTER_ADDR;    //data
    SlaveWriteBuffer[1]=0xA0;           //data
    SlaveWriteBuffer[2]=0xAB;           //data
    SlaveWriteBuffer[3]=0xAC;           //data
    SlaveCfg.tx_data = SlaveWriteBuffer;
    SlaveCfg.tx_length = 4;

  	cputs("\r\n SLAVE Write Test START\r\n");
    HAL_I2C_SlaveTransferData(Slave->pI2Cx, &SlaveCfg, I2C_TRANSFER_INTERRUPT);
    while(!complete);
    __NOP();
  	cputs("\r\n SLAVE Write Test OK \r\n");


    /* Infinite loop */
	while(1) {}
}

/*********************************************************************//**
 * @brief		Initialize buffer
 * @param[in]	buffer	buffer to initialize
 * @param[in]	type:
 * 				- 0: Initialize buffer with 0
 * 				- 1: Initialize buffer with increment value from 0
 * @return 		None
 **********************************************************************/
static void Buffer_Init(uint8_t* buffer, uint8_t type, uint16_t len) {
	uint8_t i;

	if (type) {
		for (i = 0; i < len; i++) {
			buffer[i] = i;
		}
	}
	else {
		for (i = 0; i < len; i++) {
			buffer[i] = 0;
		}
	}
}

/* --------------------------------- End Of File ------------------------------ */
