/*****************************************************************************
* @file     TestCode_TIMER1n_Oneshot.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     15. JUL. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_TIMER1n_Oneshot.h"
#include "main_conf.h"

#define MAX_TEST_STEP0_STRING		(2)
#define MAX_TEST_STEP1_STRING		(1)
#define MAX_BUFFER_LENGTH			(10)

#define ASCII_CODE_BS				(0x08) // Back space
#define ASCII_CODE_LF				(0x0A) // '\n'
#define ASCII_CODE_CR				(0x0D) // '\r'
#define ASCII_CODE_SPACE			(0x20) // Space
#define ASCII_CODE_DEL				(0x7F) // Dele

/*******************************************************************************
* Private Typedef
*******************************************************************************/
// Timer1n one-short demo configuration structure definition
typedef struct {
	// base address
	TIMER1n_Type *pTIMER1x;
 	// Output port
	PORT_Type *pOutputPort_Group;
	uint32_t OutputPort_Number;
	PCU_ALT_FUNCTION_Type OutputPort_AltFunc;
} TIMER1n_ONESHORT_Type;


/*******************************************************************************
* Private Variable
*******************************************************************************/
static const uint8_t TEST_MENU[] =
"\t - 0H: timer10(High), 0L: timer10(Low), ... 6L: timer16(Low)\r\n"
"\t - S: Start One PWM, E: Stop\r\n";

static const uint8_t PROMPT_STEP0_STRING[] =
"TIMER1n 0>";
static const uint8_t PROMPT_STEP1_STRING[] =
"TIMER1n 1>";
static const uint8_t INVALID_STRING[] =
"Invaild";

static const TIMER1n_ONESHORT_Type TIMER1n_ONESHORT_CFG[TIMER1n_IP_INDEX_MAX] = {
	{TIMER10,              PE,  6, PCU_ALT_FUNCTION_1}, // 0
	{TIMER11,              PE,  7, PCU_ALT_FUNCTION_1}, // 1
	{TIMER12,              PA,  5, PCU_ALT_FUNCTION_1}, // 2
	{TIMER13, (PORT_Type *)PF, 10, PCU_ALT_FUNCTION_1}, // 3
	{TIMER14, (PORT_Type *)PF, 11, PCU_ALT_FUNCTION_1}, // 4
	{TIMER15,              PB,  8, PCU_ALT_FUNCTION_1}, // 5
	{TIMER16,              PB,  9, PCU_ALT_FUNCTION_1}  // 6
};

static TIMER1n_CFG_Type TIMER1n_Config;


/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength);

/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* @brief      Get String
* @param      pBuffer : Pointer of string Buffer
* @param      MaxLength : Max length of buffer
* @return     String lenth
*******************************************************************************/
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength) {
	uint8_t InputData;
	uint32_t InputCount;
	static uint8_t PreInputData = 0x00;

	InputCount = 0;
	while (1) {
		InputData = cgetchar();
		if((InputData >= 0x20) && (InputData <= 0x7E)) {
			PreInputData = InputData;
			if (InputCount < MaxLength) {
				cputc(InputData);
				pBuffer[InputCount++] = InputData;
			}
		} else if ((InputData == ASCII_CODE_LF) || (InputData == ASCII_CODE_CR)) {
			if (((InputData == ASCII_CODE_LF) && (PreInputData != ASCII_CODE_CR))
				|| ((InputData == ASCII_CODE_CR) && (PreInputData != ASCII_CODE_LF))) {
				cprintf("\r\n");
				PreInputData = InputData;
                break;
			} else {
				PreInputData = InputData;
			}
		} else if ((InputData == ASCII_CODE_DEL) || (InputData == ASCII_CODE_BS)) {
			PreInputData = InputData;
			if (InputCount) {
				cputc(ASCII_CODE_BS);
				cputc(ASCII_CODE_SPACE);
				cputc(ASCII_CODE_BS);
				InputCount--;
			}
		}
	}

	return InputCount;
}
/*******************************************************************************
* @brief      TestCode_TIMER1n_Oneshot
* @param      None
* @return     None
*******************************************************************************/
void TestCode_TIMER1n_Oneshot(void) {
	uint32_t Index;
	uint32_t StringLength;
	uint8_t StringBuffer[MAX_BUFFER_LENGTH];
	const TIMER1n_ONESHORT_Type * pConfig;

    cputs((char *)TEST_MENU);

	// Timer1n and clock source setting
	HAL_SCU_Timer1n_ClockConfig(T1NCLK_PCLK);

	TIMER1n_Config.StartSync = DISABLE;
	TIMER1n_Config.ClearSync = DISABLE;
	TIMER1n_Config.ExtClock = DISABLE; // PCLK 32MHz
	TIMER1n_Config.MatchInterrupt = DISABLE;
	TIMER1n_Config.CaptureInterrupt = DISABLE;

	TIMER1n_Config.PrescalerData = 32 - 1; // 32MHz / 32 = 1MHz -> 1us

	cputs((char *)PROMPT_STEP0_STRING);

	while (1) {
		StringLength = Main_GetString(StringBuffer, MAX_BUFFER_LENGTH);
		if (StringLength == MAX_TEST_STEP0_STRING) {
			Index = (uint32_t)(StringBuffer[0] - '0');

			pConfig = &(TIMER1n_ONESHORT_CFG[Index]);

			// Timer1n output port setting
			HAL_GPIO_ConfigOutput(pConfig->pOutputPort_Group, pConfig->OutputPort_Number, PCU_MODE_ALT_FUNC);
			HAL_GPIO_ConfigFunction(pConfig->pOutputPort_Group, pConfig->OutputPort_Number, pConfig->OutputPort_AltFunc);

			if (StringBuffer[1] == 'H') {
				TIMER1n_Config.OutputPolarity = TIMER1n_OUTPUT_POLARITY_HIGH;
			} else {
				TIMER1n_Config.OutputPolarity = TIMER1n_OUTPUT_POLARITY_LOW;
			}

			TIMER1n_Config.AData = 1000; // 1ms

			TIMER1n_Config.BData = 100 * (Index + 1);

			HAL_TIMER1n_Init(pConfig->pTIMER1x, TIMER1n_ONESHOT_MODE, &TIMER1n_Config);

			cputs((char *)PROMPT_STEP1_STRING);
			while (1) {
				StringLength = Main_GetString(StringBuffer, MAX_BUFFER_LENGTH);
				if (StringLength == MAX_TEST_STEP1_STRING) {
					if (StringBuffer[0] == 'S') {
						HAL_TIMER1n_Start(pConfig->pTIMER1x);
						cputs("Start Test\r\n");
					} else if (StringBuffer[0] == 'E') {
						HAL_TIMER1n_Stop(pConfig->pTIMER1x);
						HAL_TIMER1n_DeInit(pConfig->pTIMER1x);
						cputs("Stop Test\r\n");
						cputs((char *)PROMPT_STEP0_STRING);
						break;
					} else {
						cprintf("%s\r\n", (char *)INVALID_STRING);
					}
				} else {
					cprintf("%s\r\n", (char *)INVALID_STRING);
				}
				cputs((char *)PROMPT_STEP1_STRING);
			}
		} else {
			cprintf("%s\r\n", (char *)INVALID_STRING);
			cputs((char *)PROMPT_STEP0_STRING);
		}
	}
}

/* --------------------------------- End Of File ------------------------------ */
