/*****************************************************************************
* @file     TestCode_TIMER2n_Capture.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. JUL. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_TIMER2n_Capture.h"
#include "main_conf.h"


#define MAX_TEST_STRING				(1)
#define MAX_BUFFER_LENGTH			(10)

#define ASCII_CODE_BS				(0x08) // Back space
#define ASCII_CODE_LF				(0x0A) // '\n'
#define ASCII_CODE_CR				(0x0D) // '\r'
#define ASCII_CODE_SPACE			(0x20) // Space
#define ASCII_CODE_DEL				(0x7F) // Delete

/*******************************************************************************
* Private Typedef
*******************************************************************************/
/**
 * @brief Test Time
 */
typedef enum {
	TIMER2n_TEST_TIME_00010MS,
	TIMER2n_TEST_TIME_00050MS,
	TIMER2n_TEST_TIME_00100MS,
	TIMER2n_TEST_TIME_00900MS,
	TIMER2n_TEST_TIME_01500MS,
	TIMER2n_TEST_TIME_75000MS,
	TIMER2n_TEST_TIME_00333MS,
	TIMER2n_TEST_TIME_07777MS,
	TIMER2n_TEST_TIME_09683MS,
	TIMER2n_TEST_TIME_MAX
} TIMER2n_TEST_TIME_Type;

// Timer2n capture demo configuration structure definition
typedef struct {
	// base address
	TIMER2n_Type *pTIMER2x;
	// IRQ number
	IRQn_Type TIMER2x_IRQ;
	// Capture port
	PORT_Type *pCapturePort_Group;
	uint32_t CapturePort_Number;
	PCU_ALT_FUNCTION_Type CapturePort_AltFunc;
} TIMER2n_CAPTURE_Type;


/*******************************************************************************
* Private Variable
*******************************************************************************/
static const uint8_t TEST_MENU[] =
"\t - 0: timer20, 1: timer21\r\n";

static const uint8_t PROMPT_STRING[] =
"TIMER1n>";
static const uint8_t INVALID_STRING[] =
"Invaild";

static const TIMER2n_CAPTURE_Type TIMER2n_CAPTURE_CFG[TIMER2n_IP_INDEX_MAX] = {
	{TIMER20, TIMER20_IRQn, PC, 0, PCU_ALT_FUNCTION_2},
	{TIMER21, TIMER21_IRQn, PC, 1, PCU_ALT_FUNCTION_2}
};
static const uint32_t TIMER2n_TEST_TIME[TIMER2n_TEST_TIME_MAX] = {
	25, 50, 100, 200, 400, 800, 1600, 3200, 6400
};

static Bool Timer2nFlag;
static uint32_t MilliSec;
static TIMER2n_CFG_Type TIMER2n_Config;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static void Main_Timer2nIRQHandler(TIMER2n_Type *pTIMER2x);
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength);

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_Capture(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

/*******************************************************************************
* @brief      Timer20 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER20_IRQHandler_Capture(void) {
	Main_Timer2nIRQHandler(TIMER20);
}

/*******************************************************************************
* @brief      Timer21 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER21_IRQHandler_Capture(void) {
	Main_Timer2nIRQHandler(TIMER21);
}

/*******************************************************************************
* @brief      Timer2n Interrupt Handler
* @param      None
* @return     None
*******************************************************************************/
static void Main_Timer2nIRQHandler(TIMER2n_Type *pTIMER2x) {
	if (HAL_TIMER2n_GetStatus(pTIMER2x) & TIMER2n_STATUS_CAPTURE_INT) {
		if (Timer2nFlag == FALSE) {
			Timer2nFlag = TRUE;
		}

		HAL_TIMER2n_ClearStatus(pTIMER2x, TIMER2n_STATUS_CAPTURE_INT);
	}
}


/*******************************************************************************
* @brief      Get String
* @param      pBuffer : Pointer of string Buffer
* @param      MaxLength : Max length of buffer
* @return     String lenth
*******************************************************************************/
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength) {
	uint8_t InputData;
	uint32_t InputCount;
	static uint8_t PreInputData = 0x00;

	InputCount = 0;
	while (1) {
		InputData = cgetchar();
		if((InputData >= 0x20) && (InputData <= 0x7E)) {
			PreInputData = InputData;
			if (InputCount < MaxLength) {
				cputc(InputData);
				pBuffer[InputCount++] = InputData;
			}
		} else if ((InputData == ASCII_CODE_LF) || (InputData == ASCII_CODE_CR)) {
			if (((InputData == ASCII_CODE_LF) && (PreInputData != ASCII_CODE_CR))
				|| ((InputData == ASCII_CODE_CR) && (PreInputData != ASCII_CODE_LF))) {
				_DBG("\r\n");
				PreInputData = InputData;
                break;
			} else {
				PreInputData = InputData;
			}
		} else if ((InputData == ASCII_CODE_DEL) || (InputData == ASCII_CODE_BS)) {
			PreInputData = InputData;
			if (InputCount) {
				cputc(ASCII_CODE_BS);
				cputc(ASCII_CODE_SPACE);
				cputc(ASCII_CODE_BS);
				InputCount--;
			}
		}
	}

	return InputCount;
}

/**********************************************************************
 * @brief		TestCode TIMER2n Capture Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_TIMER2n_Capture(void) {
	uint32_t Index;
	uint32_t Count;
	uint32_t StringLength;
	uint8_t StringBuffer[MAX_BUFFER_LENGTH];
	TIMER2n_TEST_TIME_Type TestTime;
	const TIMER2n_CAPTURE_Type * pConfig;

    cputs((char *)TEST_MENU);

	// Test port(PA0) setting
	HAL_GPIO_ConfigOutput(PA, 0, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PA, 0, PCU_PUPD_DISABLE);
	HAL_GPIO_SetPin(PA, _BIT(0));

	// Capture port setting
	HAL_SCU_MiscClockConfig(4, PD0_TYPE, CLKSRC_LSI, 5); // 1 clock = 10us

	// Timer2n and clock source setting
	HAL_SCU_Timer20_ClockConfig(T20CLK_PCLK);

	TIMER2n_Config.ExtClock = DISABLE; // PCLK
	TIMER2n_Config.Mode = TIMER2n_MODE_CAPTURE;
	TIMER2n_Config.CaptureSignal = TIMER2n_CAPTURE_SIGNAL_EXTERNAL;
	TIMER2n_Config.CapturePolarity = TIMER2n_CAPTURE_POLARITY_RISING;
	TIMER2n_Config.MatchInterrupt = DISABLE;
	TIMER2n_Config.CaptureInterrupt = ENABLE;

	TIMER2n_Config.AData = 0xFFFFFFFF;

	TIMER2n_Config.PrescalerData = 3200 - 1; // 32MHz / 3200 = 100kHz -> 0.1ms

	SysTick_Config(SystemCoreClock / 1000);

	__enable_irq();

	cputs((char *)PROMPT_STRING);

	while (1) {
		StringLength = Main_GetString(StringBuffer, MAX_BUFFER_LENGTH);
		if (StringLength == MAX_TEST_STRING) {
			Index = (uint32_t)(StringBuffer[0] - '0');

			pConfig = &(TIMER2n_CAPTURE_CFG[Index]);

			HAL_GPIO_ConfigOutput(pConfig->pCapturePort_Group, pConfig->CapturePort_Number, PCU_MODE_ALT_FUNC);
			HAL_GPIO_ConfigFunction(pConfig->pCapturePort_Group, pConfig->CapturePort_Number, pConfig->CapturePort_AltFunc);
			HAL_GPIO_SetDebouncePin(pConfig->pCapturePort_Group, pConfig->CapturePort_Number, ENABLE);
			HAL_GPIO_SetPin(PA, _BIT(0));
			TIMER2n_Config.CapturePolarity = TIMER2n_CAPTURE_POLARITY_FALLING;
			MilliSec = 100;
			while(MilliSec);

			NVIC_SetPriority(pConfig->TIMER2x_IRQ, 3);

			NVIC_EnableIRQ(pConfig->TIMER2x_IRQ);

			HAL_TIMER2n_Init(pConfig->pTIMER2x, &TIMER2n_Config);

			for (TestTime = TIMER2n_TEST_TIME_00010MS; TestTime < TIMER2n_TEST_TIME_MAX; TestTime++) {
				HAL_TIMER2n_ClearCounter(pConfig->pTIMER2x);
				HAL_TIMER2n_Start(pConfig->pTIMER2x);
				MilliSec = TIMER2n_TEST_TIME[TestTime];
				while(MilliSec);
				HAL_GPIO_ClearPin(PA, _BIT(0));

				cprintf("%d, ", ('0' + (uint8_t)TestTime));
				while (Timer2nFlag == FALSE);
				Timer2nFlag = FALSE;
				Count = HAL_TIMER2n_GetCaptureData(pConfig->pTIMER2x);
				cprintf("%d, ", Count);

				HAL_TIMER2n_Stop(pConfig->pTIMER2x);
				HAL_GPIO_SetPin(PA, _BIT(0));
				MilliSec = 100;
				while(MilliSec);
			}
			HAL_TIMER2n_DeInit(pConfig->pTIMER2x);
			HAL_GPIO_ClearPin(PA, _BIT(0));

			TIMER2n_Config.CapturePolarity = TIMER2n_CAPTURE_POLARITY_RISING;
			MilliSec = 100;
			while(MilliSec);

			HAL_TIMER2n_Init(pConfig->pTIMER2x, &TIMER2n_Config);

			for (TestTime = TIMER2n_TEST_TIME_00010MS; TestTime < TIMER2n_TEST_TIME_MAX; TestTime++) {
				HAL_TIMER2n_ClearCounter(pConfig->pTIMER2x);
				HAL_TIMER2n_Start(pConfig->pTIMER2x);
				MilliSec = TIMER2n_TEST_TIME[TestTime];
				while(MilliSec);
				HAL_GPIO_SetPin(PA, _BIT(0));

				cprintf("%d, ", ('0' + (uint8_t)TestTime));
				while (Timer2nFlag == FALSE);
				Timer2nFlag = FALSE;
				Count = HAL_TIMER2n_GetCaptureData(pConfig->pTIMER2x);
				HAL_TIMER2n_ClearCounter(pConfig->pTIMER2x);
				cprintf("%d, ", Count);

				HAL_TIMER2n_Stop(pConfig->pTIMER2x);
				HAL_GPIO_ClearPin(PA, _BIT(0));
				MilliSec = 100;
				while(MilliSec);
			}
			cputs(" ");
			HAL_TIMER2n_DeInit(pConfig->pTIMER2x);
			_DBG((char *)PROMPT_STRING);
		} else {
			cprintf("%s\r\n", INVALID_STRING);
			cputs((char *)PROMPT_STRING);
		}
	}
}

/* --------------------------------- End Of File ------------------------------ */
