/*****************************************************************************
* @file     TestCode_TIMER2n_Interrupt.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. JUL. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_TIMER2n_Interrupt.h"
#include "main_conf.h"

#define MAX_TEST_STRING				(2)
#define MAX_BUFFER_LENGTH			(10)

#define ASCII_CODE_BS				(0x08) // Back space
#define ASCII_CODE_LF				(0x0A) // '\n'
#define ASCII_CODE_CR				(0x0D) // '\r'
#define ASCII_CODE_SPACE			(0x20) // Space
#define ASCII_CODE_DEL				(0x7F) // Delete

/*******************************************************************************
* Private Typedef
*******************************************************************************/
// Timer2n match demo configuration structure definition
typedef struct {
	// base address
	TIMER2n_Type *pTIMER2x;
	// IRQ number
	IRQn_Type TIMER2x_IRQ;
	// Output port
	PORT_Type *pOutputPort_Group;
	uint32_t OutputPort_Number;
	PCU_ALT_FUNCTION_Type OutputPort_AltFunc;
	// External clock port
	PORT_Type *pExtClockPort_Group;
	uint32_t ExtClockPort_Number;
	PCU_ALT_FUNCTION_Type ExtClockPort_AltFunc;
} TIMER2n_MATCH_Type;

/*******************************************************************************
* Private Variable
*******************************************************************************/
static const uint8_t PROMPT_STRING[] =
"TIMER2n>";
static const uint8_t INVALID_STRING[] =
"Invaild";

static const TIMER2n_MATCH_Type TIMER2n_MATCH_CFG[TIMER2n_IP_INDEX_MAX] = {
	{TIMER20, TIMER20_IRQn, PC, 0, PCU_ALT_FUNCTION_1, PC, 2, PCU_ALT_FUNCTION_1},
	{TIMER21, TIMER21_IRQn, PC, 1, PCU_ALT_FUNCTION_1, PC, 3, PCU_ALT_FUNCTION_1}
};

static Bool Timer2nFlag;
static uint32_t Timer2nCount;

static TIMER1n_CFG_Type TIMER1n_Config;
static TIMER2n_CFG_Type TIMER2n_Config;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static void Main_InitializeTimer1n(void);
static void Main_Timer2nIRQHandler(TIMER2n_Type *pTIMER2x);
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength);

/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* @brief      Timer20 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER20_IRQHandler_Interrupt(void) {
	Main_Timer2nIRQHandler(TIMER20);
}

/*******************************************************************************
* @brief      Timer21 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER21_IRQHandler_Interrupt(void) {
	Main_Timer2nIRQHandler(TIMER21);
}

/*******************************************************************************
* @brief      Timer10 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER10_IRQHandler_Interrupt(void) {
	if (HAL_TIMER1n_GetStatus(TIMER10) & TIMER1n_STATUS_MATCH_INT) {
		HAL_TIMER1n_ClearStatus(TIMER10, TIMER1n_STATUS_MATCH_INT);
	}
}

/*******************************************************************************
* @brief      Timer2n Interrupt Handler
* @param      None
* @return     None
*******************************************************************************/
static void Main_Timer2nIRQHandler(TIMER2n_Type *pTIMER2x) {
	if (HAL_TIMER2n_GetStatus(pTIMER2x) & TIMER2n_STATUS_MATCH_INT) {
		Timer2nCount++;
		if (Timer2nCount >= 200) {
			Timer2nCount = 0;

			if (Timer2nFlag == TRUE) {
				Timer2nFlag = FALSE;
				HAL_GPIO_ClearPin(PE, _BIT(2));
			} else {
				Timer2nFlag = TRUE;
				HAL_GPIO_SetPin(PE, _BIT(2));
			}
		}

		HAL_TIMER2n_ClearStatus(pTIMER2x, TIMER2n_STATUS_MATCH_INT);
	}
}

/*******************************************************************************
* @brief      Initialize Timer1n for External Clock Source
* @param      None
* @return     None
*******************************************************************************/
static void Main_InitializeTimer1n(void) {
	// Timer1n output pin setting
	HAL_GPIO_ConfigOutput(PE, 6, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 6, PCU_ALT_FUNCTION_1);

	HAL_SCU_Timer1n_ClockConfig(T1NCLK_PCLK);

	TIMER1n_Config.ExtClock = DISABLE; // PCLK 48MHz
	TIMER1n_Config.PrescalerData = 24 - 1; // 48MHz / 24 = 2MHz -> 0.5us
	TIMER1n_Config.OutputPolarity = TIMER1n_OUTPUT_POLARITY_LOW;
	TIMER1n_Config.MatchInterrupt = ENABLE;
	TIMER1n_Config.CaptureInterrupt = DISABLE;

	TIMER1n_Config.AData = 4 - 1; // 2us

	HAL_TIMER1n_Init(TIMER10, TIMER1n_PERIODIC_MODE, &TIMER1n_Config);

	NVIC_SetPriority(TIMER10_IRQn, 3);
	NVIC_EnableIRQ(TIMER10_IRQn);

	HAL_TIMER1n_Start(TIMER10);
}

/*******************************************************************************
* @brief      Get String
* @param      pBuffer : Pointer of string Buffer
* @param      MaxLength : Max length of buffer
* @return     String lenth
*******************************************************************************/
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength) {
	uint8_t InputData;
	uint32_t InputCount;
	static uint8_t PreInputData = 0x00;

	InputCount = 0;
	while (1) {
		InputData = cgetchar();
		if((InputData >= 0x20) && (InputData <= 0x7E)) {
			PreInputData = InputData;
			if (InputCount < MaxLength) {
				cputc(InputData);
				pBuffer[InputCount++] = InputData;
			}
		} else if ((InputData == ASCII_CODE_LF) || (InputData == ASCII_CODE_CR)) {
			if (((InputData == ASCII_CODE_LF) && (PreInputData != ASCII_CODE_CR))
				|| ((InputData == ASCII_CODE_CR) && (PreInputData != ASCII_CODE_LF))) {
				cprintf("\r\n");
				PreInputData = InputData;
                break;
			} else {
				PreInputData = InputData;
			}
		} else if ((InputData == ASCII_CODE_DEL) || (InputData == ASCII_CODE_BS)) {
			PreInputData = InputData;
			if (InputCount) {
				cputc(ASCII_CODE_BS);
				cputc(ASCII_CODE_SPACE);
				cputc(ASCII_CODE_BS);
				InputCount--;
			}
		}
	}

	return InputCount;
}

/**********************************************************************
 * @brief		TestCode TIMER2n Interrupt Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_TIMER2n_Interrupt(void) {
	Bool StartedTest;
	uint32_t Index;
	uint32_t StringLength;
	const TIMER2n_MATCH_Type * pConfig;
	uint8_t StringBuffer[MAX_BUFFER_LENGTH];

	Timer2nFlag = FALSE;
	Timer2nCount = 0;

	// Test port(PE0) setting
	HAL_GPIO_ConfigOutput(PE, 2, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 2, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(2));

	__enable_irq();

	Main_InitializeTimer1n();

	cputs((char *)PROMPT_STRING);
	StartedTest = FALSE;

	while (1) {
		StringLength = Main_GetString(StringBuffer, MAX_BUFFER_LENGTH);
		if (StringLength == MAX_TEST_STRING) {
			if (StartedTest == TRUE) {
				pConfig = &(TIMER2n_MATCH_CFG[Index]);
				HAL_TIMER2n_Stop(pConfig->pTIMER2x);
				NVIC_DisableIRQ(pConfig->TIMER2x_IRQ);
				HAL_TIMER2n_DeInit(pConfig->pTIMER2x);
			} else {
				StartedTest = TRUE;
			}

			Index = (uint32_t)(StringBuffer[0] - '0');
			pConfig = &(TIMER2n_MATCH_CFG[Index]);

			// Timer2n output port setting
			HAL_GPIO_ConfigOutput(pConfig->pOutputPort_Group, pConfig->OutputPort_Number, PCU_MODE_ALT_FUNC);
			HAL_GPIO_ConfigFunction(pConfig->pOutputPort_Group, pConfig->OutputPort_Number, pConfig->OutputPort_AltFunc);

			// Timer2n and clock source setting (MCCR1 / PCLK / ECn)
			if (StringBuffer[1] == 'M') {
				// only timer20
				HAL_SCU_MiscClockConfig(2, TIMER2n_TYPE, CLKSRC_MCLK, 2);  // 32/2 = 16MHz
				HAL_SCU_Timer20_ClockConfig(T20CLK_MCCR2);

				TIMER2n_Config.ExtClock = DISABLE; // MCCR 16MHz
				TIMER2n_Config.PrescalerData = 16 - 1; // 16MHz / 16 = 1MHz -> 1us
			} else if (StringBuffer[1] == 'P') {
				HAL_SCU_Timer20_ClockConfig(T20CLK_PCLK);

				TIMER2n_Config.ExtClock = DISABLE; // PCLK 32MHz
				TIMER2n_Config.PrescalerData = 64 - 1; // 32MHz / 64 = 0.5MHz -> 2us
			} else {
				HAL_GPIO_ConfigOutput(pConfig->pExtClockPort_Group, pConfig->ExtClockPort_Number, PCU_MODE_ALT_FUNC);
				HAL_GPIO_ConfigFunction(pConfig->pExtClockPort_Group, pConfig->ExtClockPort_Number, pConfig->ExtClockPort_AltFunc);

				TIMER2n_Config.ExtClock = ENABLE; // EC
				TIMER2n_Config.ExtClockEdge = TIMER2n_EXT_CLOCK_EDGE_FALLING;
				TIMER2n_Config.PrescalerData = 0; // Bypass
			}

			TIMER2n_Config.Mode = TIMER2n_MODE_PERIODIC;
			TIMER2n_Config.OutputPolarity = TIMER2n_OUTPUT_POLARITY_LOW;
			TIMER2n_Config.MatchInterrupt = ENABLE;
			TIMER2n_Config.CaptureInterrupt = DISABLE;

			TIMER2n_Config.AData = 1000 - 1; // 1ms

			HAL_TIMER2n_Init(pConfig->pTIMER2x, &TIMER2n_Config);

			NVIC_SetPriority(pConfig->TIMER2x_IRQ, 3);

			NVIC_EnableIRQ(pConfig->TIMER2x_IRQ);

			HAL_TIMER2n_Start(pConfig->pTIMER2x);
		}else {
			cprintf("%s\r\n", INVALID_STRING);
		}
		cputs((char *)PROMPT_STRING);
	}
}

/* --------------------------------- End Of File ------------------------------ */
