/*****************************************************************************
* @file     TestCode_TIMER2n_Oneshot.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. JUL. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_TIMER2n_Oneshot.h"
#include "main_conf.h"


#define MAX_TEST_STEP0_STRING		(2)
#define MAX_TEST_STEP1_STRING		(1)
#define MAX_BUFFER_LENGTH			(10)

#define ASCII_CODE_BS				(0x08) // Back space
#define ASCII_CODE_LF				(0x0A) // '\n'
#define ASCII_CODE_CR				(0x0D) // '\r'
#define ASCII_CODE_SPACE			(0x20) // Space
#define ASCII_CODE_DEL				(0x7F) // Delete


/*******************************************************************************
* Private Typedef
*******************************************************************************/
// Timer2n one-short demo configuration structure definition
typedef struct {
	// base address
	TIMER2n_Type *pTIMER2x;
 	// Output port
	PORT_Type *pOutputPort_Group;
	uint32_t OutputPort_Number;
	PCU_ALT_FUNCTION_Type OutputPort_AltFunc;
} TIMER2n_ONESHORT_Type;



/*******************************************************************************
* Private Variable
*******************************************************************************/
static const uint8_t TEST_MENU[] =
"\t - 0H: timer20(High), 0L: timer20(Low)\r\n"
"\t - S: Start One PWM, E: Stop\r\n";

static const uint8_t PROMPT_STEP0_STRING[] =
"TIMER2n 0>";
static const uint8_t PROMPT_STEP1_STRING[] =
"TIMER2n 1>";
static const uint8_t INVALID_STRING[] =
"Invaild";

static const TIMER2n_ONESHORT_Type TIMER2n_ONESHORT_CFG[TIMER2n_IP_INDEX_MAX] = {
	{TIMER20, PC, 0, PCU_ALT_FUNCTION_1},
	{TIMER21, PC, 1, PCU_ALT_FUNCTION_1}
};

static TIMER2n_CFG_Type TIMER2n_Config;


/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength);

/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* @brief      Get String
* @param      pBuffer : Pointer of string Buffer
* @param      MaxLength : Max length of buffer
* @return     String lenth
*******************************************************************************/
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength) {
	uint8_t InputData;
	uint32_t InputCount;
	static uint8_t PreInputData = 0x00;

	InputCount = 0;
	while (1) {
		InputData = cgetchar();
		if((InputData >= 0x20) && (InputData <= 0x7E)) {
			PreInputData = InputData;
			if (InputCount < MaxLength) {
				cputc(InputData);
				pBuffer[InputCount++] = InputData;
			}
		} else if ((InputData == ASCII_CODE_LF) || (InputData == ASCII_CODE_CR)) {
			if (((InputData == ASCII_CODE_LF) && (PreInputData != ASCII_CODE_CR))
				|| ((InputData == ASCII_CODE_CR) && (PreInputData != ASCII_CODE_LF))) {
				_DBG("\r\n");
				PreInputData = InputData;
                break;
			} else {
				PreInputData = InputData;
			}
		} else if ((InputData == ASCII_CODE_DEL) || (InputData == ASCII_CODE_BS)) {
			PreInputData = InputData;
			if (InputCount) {
				cputc(ASCII_CODE_BS);
				cputc(ASCII_CODE_SPACE);
				cputc(ASCII_CODE_BS);
				InputCount--;
			}
		}
	}

	return InputCount;
}

/**********************************************************************
 * @brief		TestCode TIMER2n Oneshot Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_TIMER2n_Oneshot(void) {
	uint32_t Index;
	uint32_t StringLength;
	uint8_t StringBuffer[MAX_BUFFER_LENGTH];
	const TIMER2n_ONESHORT_Type * pConfig;

    cputs((char *)TEST_MENU);

	// Test port(PE0) setting
	HAL_GPIO_ConfigOutput(PE, 2, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 2, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(2));

	HAL_SCU_Timer20_ClockConfig(T20CLK_PCLK);

	TIMER2n_Config.ExtClock = DISABLE; // PCLK 32MHz
	TIMER2n_Config.PrescalerData = 32 - 1; // 32MHz / 32 = 1MHz -> 1us
	TIMER2n_Config.Mode = TIMER2n_MODE_ONESHOT;
	TIMER2n_Config.MatchInterrupt = DISABLE;
	TIMER2n_Config.CaptureInterrupt = DISABLE;

	cputs((char *)PROMPT_STEP0_STRING);

	while (1) {
		StringLength = Main_GetString(StringBuffer, MAX_BUFFER_LENGTH);
		if (StringLength == MAX_TEST_STEP0_STRING) {
			Index = (uint32_t)(StringBuffer[0] - '0');

			pConfig = &(TIMER2n_ONESHORT_CFG[Index]);

			// Timer2n output port setting
			HAL_GPIO_ConfigOutput(pConfig->pOutputPort_Group, pConfig->OutputPort_Number, PCU_MODE_ALT_FUNC);
			HAL_GPIO_ConfigFunction(pConfig->pOutputPort_Group, pConfig->OutputPort_Number, pConfig->OutputPort_AltFunc);

			if (StringBuffer[1] == 'H') {
				TIMER2n_Config.OutputPolarity = TIMER2n_OUTPUT_POLARITY_HIGH;
			} else {
				TIMER2n_Config.OutputPolarity = TIMER2n_OUTPUT_POLARITY_LOW;
			}

			TIMER2n_Config.AData = 1000; // 1ms

			TIMER2n_Config.BData = 100 * (Index + 1);

			HAL_TIMER2n_Init(pConfig->pTIMER2x, &TIMER2n_Config);

			cputs((char *)PROMPT_STEP1_STRING);
			while (1) {
				StringLength = Main_GetString(StringBuffer, MAX_BUFFER_LENGTH);
				if (StringLength == MAX_TEST_STEP1_STRING) {
					if (StringBuffer[0] == 'S') {
						HAL_TIMER2n_Start(pConfig->pTIMER2x);
						cputs("Start Test\r\n");
					} else if (StringBuffer[0] == 'E') {
						HAL_TIMER2n_Stop(pConfig->pTIMER2x);
						HAL_TIMER2n_DeInit(pConfig->pTIMER2x);
						cputs("Stop Test\r\n");
						cputs((char *)PROMPT_STEP0_STRING);
						break;
					} else {
						cprintf("%s\r\n", INVALID_STRING);
					}
				} else {
					cprintf("%s\r\n", INVALID_STRING);
				}
				cputs((char *)PROMPT_STEP1_STRING);
			}
		} else {
			cprintf("%s\r\n", INVALID_STRING);
			cputs((char *)PROMPT_STEP0_STRING);
		}
	}
}

/* --------------------------------- End Of File ------------------------------ */
