/*****************************************************************************
* @file     TestCode_TIMER2n_Polling.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. JUL. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_TIMER2n_Polling.h"
#include "main_conf.h"

#define MAX_TEST_STRING				(1)
#define MAX_BUFFER_LENGTH			(10)

#define ASCII_CODE_BS				(0x08) // Back space
#define ASCII_CODE_LF				(0x0A) // '\n'
#define ASCII_CODE_CR				(0x0D) // '\r'
#define ASCII_CODE_SPACE			(0x20) // Space
#define ASCII_CODE_DEL				(0x7F) // Delete

/*******************************************************************************
* Private Typedef
*******************************************************************************/
// Timer2n match demo configuration structure definition
typedef struct {
	// base address
	TIMER2n_Type *pTIMER2x;
	// IRQ number
	IRQn_Type TIMER2x_IRQ;
	// Output port
	PORT_Type *pOutputPort_Group;
	uint32_t OutputPort_Number;
	PCU_ALT_FUNCTION_Type OutputPort_AltFunc;
} TIMER2n_MATCH_Type;

/*******************************************************************************
* Private Variable
*******************************************************************************/
static const uint8_t TEST_MENU[] =
"\t - 0: timer20, 1: timer21\r\n";
static const uint8_t PROMPT_STRING[] =
"TIMER2n>";
static const uint8_t INVALID_STRING[] =
"Invaild";

const TIMER2n_MATCH_Type TIMER2n_MATCH_CFG[TIMER2n_IP_INDEX_MAX] = {
	{TIMER20, TIMER20_IRQn, PC, 0, PCU_ALT_FUNCTION_1},
	{TIMER21, TIMER21_IRQn, PC, 1, PCU_ALT_FUNCTION_1}
};

TIMER2n_CFG_Type TIMER2n_Config;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength);
/*******************************************************************************
* Public Function
*******************************************************************************/


/*******************************************************************************
* @brief      Get String
* @param      pBuffer : Pointer of string Buffer
* @param      MaxLength : Max length of buffer
* @return     String lenth
*******************************************************************************/
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength) {
	uint8_t InputData;
	uint32_t InputCount;
	static uint8_t PreInputData = 0x00;

	InputCount = 0;
	while (1) {
		InputData = cgetchar();
		if((InputData >= 0x20) && (InputData <= 0x7E)) {
			PreInputData = InputData;
			if (InputCount < MaxLength) {
				cputc(InputData);
				pBuffer[InputCount++] = InputData;
			}
		} else if ((InputData == ASCII_CODE_LF) || (InputData == ASCII_CODE_CR)) {
			if (((InputData == ASCII_CODE_LF) && (PreInputData != ASCII_CODE_CR))
				|| ((InputData == ASCII_CODE_CR) && (PreInputData != ASCII_CODE_LF))) {
				_DBG("\r\n");
				PreInputData = InputData;
                break;
			} else {
				PreInputData = InputData;
			}
		} else if ((InputData == ASCII_CODE_DEL) || (InputData == ASCII_CODE_BS)) {
			PreInputData = InputData;
			if (InputCount) {
				cputc(ASCII_CODE_BS);
				cputc(ASCII_CODE_SPACE);
				cputc(ASCII_CODE_BS);
				InputCount--;
			}
		}
	}

	return InputCount;
}

/**********************************************************************
 * @brief		TestCode TIMER2n Polling Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_TIMER2n_Polling(void) {
	Bool Flag;
	uint32_t Index;
	uint32_t Count;
	uint32_t StringLength;
	const TIMER2n_MATCH_Type *pConfig;
	uint8_t StringBuffer[MAX_BUFFER_LENGTH];

    cputs((char *)TEST_MENU);

	// Test port(PE0) setting
	HAL_GPIO_ConfigOutput(PE, 2, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 2, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(2));

#if 0
    // TIMER2n uses HSE clock source
    HAL_SCU_HSE_ClockConfig(HSE_EN);
    HAL_SCU_MiscClockConfig(2, TIMER2n_TYPE, CLKSRC_HSE, 1);   // 8MHz
    TIMER2n_Config.PrescalerData = 8 - 1; // 8MHz / 8 = 1MHz -> 1us
    HAL_SCU_Timer20_ClockConfig(T20CLK_MCCR2);
#endif
#if 1
    // TIMER2n uses HSI clock source
    // HAL_SCU_HSI_ClockConfig(HSI_EN);  // If HSI clock source is still used as MCLK, this function doesn't need to be set.
    HAL_SCU_MiscClockConfig(2, TIMER2n_TYPE, CLKSRC_HSI, 1);   // 32MHz
    TIMER2n_Config.PrescalerData = 32 - 1; // 32MHz / 32 = 1MHz -> 1us
    HAL_SCU_Timer20_ClockConfig(T20CLK_MCCR2);
#endif
#if 0
    // TIMER2n uses PCLK clock source
    TIMER2n_Config.PrescalerData = 32 - 1; // 32MHz / 32 = 1MHz -> 1us
    HAL_SCU_Timer20_ClockConfig(T20CLK_PCLK);
#endif
#if 0
    // TIMER2n uses LSI clock source
//    SCU_SetLSI(LSI_EN);   // LSI clock always on system (
    HAL_SCU_MiscClockConfig(2, TIMER2n_TYPE, CLKSRC_LSI, 1);   // 500kHz
    TIMER2n_Config.PrescalerData = 5 - 1; // 500kHz / 5 = 100kHz
    HAL_SCU_Timer20_ClockConfig(T20CLK_MCCR2);
#endif
#if 0
    // TIMER2n uses LSE clock source
    HAL_SCU_LSE_ClockConfig(LSE_EN);
    HAL_SCU_MiscClockConfig(2, TIMER2n_TYPE, CLKSRC_LSE, 1);   // 32.768
    TIMER2n_Config.PrescalerData = 32 - 1;  // 32768/32 = 1024Hz
    HAL_SCU_Timer20_ClockConfig(T20CLK_MCCR2);
#endif

    TIMER2n_Config.ExtClock = DISABLE;
	TIMER2n_Config.Mode = TIMER2n_MODE_PERIODIC;
	TIMER2n_Config.OutputPolarity = TIMER2n_OUTPUT_POLARITY_LOW;
	TIMER2n_Config.MatchInterrupt = DISABLE;
	TIMER2n_Config.CaptureInterrupt = DISABLE;

	Flag = FALSE;
	Count = 0;

	cputs((char *)PROMPT_STRING);

	while (1) {
		StringLength = Main_GetString(StringBuffer, MAX_BUFFER_LENGTH);
		if (StringLength == MAX_TEST_STRING) {
			Index = (uint32_t)(StringBuffer[0] - '0');

			pConfig = &(TIMER2n_MATCH_CFG[Index]);

			// Timer2n output port setting
			HAL_GPIO_ConfigOutput(pConfig->pOutputPort_Group, pConfig->OutputPort_Number, PCU_MODE_ALT_FUNC);
			HAL_GPIO_ConfigFunction(pConfig->pOutputPort_Group, pConfig->OutputPort_Number, pConfig->OutputPort_AltFunc);

			TIMER2n_Config.AData = (1000 * (Index + 1)) - 1;

			HAL_TIMER2n_Init(pConfig->pTIMER2x, &TIMER2n_Config);

			HAL_TIMER2n_Start(pConfig->pTIMER2x);
			cputs("Test Start\r\n");

			while (1) {
				while ((HAL_TIMER2n_GetStatus(pConfig->pTIMER2x) & TIMER2n_STATUS_MATCH_INT) == 0);
				HAL_TIMER2n_ClearStatus(pConfig->pTIMER2x, TIMER2n_STATUS_MATCH_INT);

				Count++;
				if (Count >= 200) {
					Count = 0;

					if (Flag == TRUE) {
						Flag = FALSE;
						cputs("Off\r\n");
						HAL_GPIO_ClearPin(PE, _BIT(2));
					} else {
						Flag = TRUE;
						cputs("On\r\n");
						HAL_GPIO_SetPin(PE, _BIT(2));
					}
				}
			}
		} else {
			cputs((char *)INVALID_STRING);
			cputs((char *)PROMPT_STRING);
		}
	}
}

/* --------------------------------- End Of File ------------------------------ */
