/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* buffer size definition */
#define UART_RING_BUFSIZE 32

/* Buf mask */
#define __BUF_MASK (UART_RING_BUFSIZE-1)
/* Check buf is full or not */
#define __BUF_IS_FULL(head, tail) ((tail&__BUF_MASK)==((head+1)&__BUF_MASK))
/* Check buf will be full in next receiving or not */
#define __BUF_WILL_FULL(head, tail) ((tail&__BUF_MASK)==((head+2)&__BUF_MASK))
/* Check buf is empty */
#define __BUF_IS_EMPTY(head, tail) ((head&__BUF_MASK)==(tail&__BUF_MASK))
/* Reset buf */
#define __BUF_INCR(bufidx)	(bufidx=(bufidx+1)&__BUF_MASK)

/* Private function prototypes ---------------------------------------*/

void UART1_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void UART_IntReceive(UART_Type *UARTx);
void UART_IntTransmit(UART_Type *UARTx);
void UART_IntErr(uint8_t bLSErrType);
uint32_t UARTSend(UART_Type *UARTx, uint8_t *txbuf, uint8_t buflen);
uint32_t UARTReceive(UART_Type *UARTx, uint8_t *rxbuf, uint8_t buflen);
void GPIO_Configure(void);
void UART1_Configure(void);
void UART1_SendRun(void);
void UART1_ReceivRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/
typedef struct
{
    uint32_t tx_head;                /*!< UART Tx ring buffer head index */
    uint32_t tx_tail;                /*!< UART Tx ring buffer tail index */
    uint32_t rx_head;                /*!< UART Rx ring buffer head index */
    uint32_t rx_tail;                /*!< UART Rx ring buffer tail index */
    uint8_t  tx[UART_RING_BUFSIZE];  /*!< UART Tx data ring buffer */
    uint8_t  rx[UART_RING_BUFSIZE];  /*!< UART Rx data ring buffer */
} UART_RING_BUFFER_T;
//uint32_t msec;
//const uint8_t menu[] =
//"************************************************\n\r"
//" ADC demo \n\r"
//"\t - MCU: A31G22x \n\r"
//"\t - Core: ARM Cortex-M0+ \n\r"
//"\t - Communicate via: UART1 - 38400 bps \n\r"
//" Using ADC in interrupt mode \n\r"
//"************************************************\n\r";

UART_CFG_Type UARTConfigStruct;
// UART Ring buffer
UART_RING_BUFFER_T rb;

// Current Tx Interrupt enable state
__IO FlagStatus TxIntStat;

/*********************************************************************
 * @brief		UART1 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void UART1_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp, tmp1;

	/* Determine the interrupt source */
	intsrc = UART1->IIR;
	tmp = intsrc & UART_IIR_INTID_MASK;

	// Receive Line Status
	if (tmp == UART_IIR_INTID_RLS){
		// Check line status
		tmp1 = UART1->LSR;
		// Mask out the Receive Ready and Transmit Holding empty status
		tmp1 &= (UART_LSR_OE | UART_LSR_PE | UART_LSR_FE | UART_LSR_BI);
		// If any error exist
		if (tmp1) {
				UART_IntErr(tmp1);
		}
	}

	// Receive Data Available or Character time-out
	if (tmp == UART_IIR_INTID_RDA){
			UART_IntReceive(UART1);
	}

	// Transmit Holding Empty
	if (tmp == UART_IIR_INTID_THRE){
			UART_IntTransmit(UART1);
	}
}




/********************************************************************//**
 * @brief 		UART receive function (ring buffer used)
 * @param[in]	None
 * @return 		None
 *********************************************************************/
void UART_IntReceive(UART_Type *UARTx)
{
	uint8_t tmpc;
	uint32_t rLen;

	while(1){
		// Call UART read function in UART driver
		rLen = HAL_UART_Receive((UART_Type *)UARTx, &tmpc, 1, NONE_BLOCKING);
		// If data received
		if (rLen){
			/* Check if buffer is more space
			 * If no more space, remaining character will be trimmed out
			 */
			if (!__BUF_IS_FULL(rb.rx_head,rb.rx_tail)){
				rb.rx[rb.rx_head] = tmpc;
				__BUF_INCR(rb.rx_head);
			}
		}
		// no more data
		else {
			break;
		}
	}
}

/********************************************************************//**
 * @brief 		UART transmit function (ring buffer used)
 * @param[in]	None
 * @return 		None
 *********************************************************************/
void UART_IntTransmit(UART_Type *UARTx)
{
    // Disable THRE interrupt
    HAL_UART_ConfigInterrupt((UART_Type *)UARTx, UART_INTCFG_THRE, DISABLE);

	/* Wait until THR empty */
    while (HAL_UART_CheckBusy((UART_Type *)UARTx) == SET);

	while (!__BUF_IS_EMPTY(rb.tx_head,rb.tx_tail))
    {
        /* Move a piece of data into the transmit FIFO */
    	if (HAL_UART_Transmit((UART_Type *)UARTx, (uint8_t *)&rb.tx[rb.tx_tail], 1, NONE_BLOCKING)){
        /* Update transmit ring FIFO tail pointer */
        __BUF_INCR(rb.tx_tail);
    	} else {
    		break;
    	}
    }

    /* If there is no more data to send, disable the transmit
       interrupt - else enable it or keep it enabled */
	if (__BUF_IS_EMPTY(rb.tx_head, rb.tx_tail)) {
    	HAL_UART_ConfigInterrupt((UART_Type *)UARTx, UART_INTCFG_THRE, DISABLE);
    	// Reset Tx Interrupt state
    	TxIntStat = RESET;
    }
    else
		{
      	// Set Tx Interrupt state
		TxIntStat = SET;
    	HAL_UART_ConfigInterrupt((UART_Type *)UARTx, UART_INTCFG_THRE, ENABLE);
    }
}
/*********************************************************************//**
 * @brief		UART Line Status Error
 * @param[in]	bLSErrType	UART Line Status Error Type
 * @return		None
 **********************************************************************/
void UART_IntErr(uint8_t bLSErrType)
{
	uint8_t test;
	// Loop forever
	while (1){
		// For testing purpose
		test = bLSErrType;
		test = test;
	}
}
/*********************************************************************
 * @brief		UART transmit function for interrupt mode (using ring buffers)
 * @param[in]	UARTPort	Selected UART peripheral used to send data,
 * 				should be UART1
 * @param[out]	txbuf Pointer to Transmit buffer
 * @param[in]	buflen Length of Transmit buffer
 * @return 		Number of bytes actually sent to the ring buffer
 **********************************************************************/
uint32_t UARTSend(UART_Type *UARTx, uint8_t *txbuf, uint8_t buflen)
{
    uint8_t *data = (uint8_t *) txbuf;
    uint32_t bytes = 0;

	/* Temporarily lock out UART transmit interrupts during this
	   read so the UART transmit interrupt won't cause problems
	   with the index values */
    HAL_UART_ConfigInterrupt(UARTx, UART_INTCFG_THRE, DISABLE);

	/* Loop until transmit run buffer is full or until n_bytes
	   expires */
	while ((buflen > 0) && (!__BUF_IS_FULL(rb.tx_head, rb.tx_tail)))
	{
		/* Write data from buffer into ring buffer */
		rb.tx[rb.tx_head] = *data;
		data++;

		/* Increment head pointer */
		__BUF_INCR(rb.tx_head);

		/* Increment data count and decrement buffer size count */
		bytes++;
		buflen--;
	}

	/*
	 * Check if current Tx interrupt enable is reset,
	 * that means the Tx interrupt must be re-enabled
	 * due to call UART_IntTransmit() function to trigger
	 * this interrupt type
	 */
	if (TxIntStat == RESET) {
		UART_IntTransmit(UARTx);
	}
	/*
	 * Otherwise, re-enables Tx Interrupt
	 */
	else {
		HAL_UART_ConfigInterrupt(UARTx, UART_INTCFG_THRE, ENABLE);
	}

    return bytes;
}
/*********************************************************************
 * @brief		UART read function for interrupt mode (using ring buffers)
 * @param[in]	UARTPort	Selected UART peripheral used to send data,
 * 				should be UART1
 * @param[out]	rxbuf Pointer to Received buffer
 * @param[in]	buflen Length of Received buffer
 * @return 		Number of bytes actually read from the ring buffer
 **********************************************************************/
uint32_t UARTReceive(UART_Type *UARTx, uint8_t *rxbuf, uint8_t buflen)
{
    uint8_t *data = (uint8_t *) rxbuf;
    uint32_t bytes = 0;

	/* Temporarily lock out UART receive interrupts during this
	   read so the UART receive interrupt won't cause problems
	   with the index values */
	HAL_UART_ConfigInterrupt(UARTx, UART_INTCFG_RBR, DISABLE);

	/* Loop until receive buffer ring is empty or
		until max_bytes expires */
	while ((buflen > 0) && (!(__BUF_IS_EMPTY(rb.rx_head, rb.rx_tail))))
	{
		/* Read data from ring buffer into user buffer */
		*data = rb.rx[rb.rx_tail];
		data++;

		/* Update tail pointer */
		__BUF_INCR(rb.rx_tail);

		/* Increment data count and decrement buffer size count */
		bytes++;
		buflen--;
	}

	/* Re-enable UART interrupts */
	HAL_UART_ConfigInterrupt(UARTx, UART_INTCFG_RBR, ENABLE);

    return bytes;
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
    HAL_GPIO_ConfigOutput(PB, 7, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigFunction(PB, 7, PCU_ALT_FUNCTION_1);
	HAL_GPIO_ConfigPullup(PB, 7, PCU_PUPD_PULL_UP);

    HAL_GPIO_ConfigOutput(PB, 6, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigFunction(PB, 6, PCU_ALT_FUNCTION_1);
}

/**********************************************************************
 * @brief		UART1_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/

#define	TEST_TX_SIZE	32
#define	TEST_RX_SIZE	32
static uint8_t	test_buffer[TEST_RX_SIZE + TEST_TX_SIZE];

void UART1_Configure(void)
{
	/*
	 * Initialize UART1 pin connect
	 */
	HAL_GPIO_ConfigOutput(PB, 7, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PB, 7, PCU_ALT_FUNCTION_1);
	HAL_GPIO_ConfigPullup(PB, 7, PCU_PUPD_DISABLE);
	
	HAL_GPIO_ConfigOutput(PB, 6, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PB, 6, PCU_ALT_FUNCTION_1);


	/* Initialize UART Configuration parameter structure to default state:
	 * Baudrate = 38400bps
	 * 8 data bit
	 * 1 Stop bit
	 * None parity
	 */
	HAL_UART_ConfigStructInit(&UARTConfigStruct);
    
	UARTConfigStruct.RxTxBuffer = test_buffer;
	UARTConfigStruct.RxBufferSize = TEST_RX_SIZE;
	UARTConfigStruct.TxBufferSize = TEST_TX_SIZE;
	
    uart_set_buffer(UART1, test_buffer, UARTConfigStruct.RxBufferSize, UARTConfigStruct.TxBufferSize);
    
	// Initialize DEBUG_UART_PORT peripheral with given to corresponding parameter
	HAL_UART_Init((UART_Type*)DEBUG_PORT, &UARTConfigStruct);

	NVIC_SetPriority(UART1_IRQn, 3);
	NVIC_EnableIRQ(UART1_IRQn);
    __enable_irq();

}
	

/**********************************************************************
 * @brief		UART1_SendRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART1_SendRun(void)
{	
    uputs(UART1, (char *)"Interrupt test \n\r");
}

/**********************************************************************
 * @brief		UART1_ReceivRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART1_ReceivRun(void)
{
	uint32_t idx, len;
	uint8_t buffer[11];
	
	len = 0;
	while (len == 0)
	{
		len = ugets((UART_Type *)UART1, (char *)buffer, sizeof(buffer));    // get data buffer
	}

	/* Got some data */
    if (buffer[idx] == 0x0d)
    {
        uputs((UART_Type *)UART1, "\r\n");
    }
    else
    {
        /* Echo it back */
        uputs((UART_Type *)UART1, (char *)&buffer);                         // the last data of buffer is '0'
    }
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{

	/*Configure port peripheral*/
	GPIO_Configure();  
	
	/*Configure UART1 peripheral*/
	UART1_Configure();
	
	/* Enable IRQ Interrupts */
	__enable_irq();    
    
	/*UART1 send*/
	UART1_SendRun();
  
	while(1)
	{
   /* Infinite loop */
		UART1_ReceivRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
