/*****************************************************************************
* @file     TestCode_WDT_SleepWakeup.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     12. JUN. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_WDT_SleepWakeup.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint32_t MilliSec;       // Systick Counter Value

static uint8_t wdtf_uf;  		// underflow flag
static uint8_t wdtf_wmf; 		// window match flag
static int gCnt = 0;
static uint32_t WakeupFlag;     // Wakeup flag

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_TestSleepAndWdtWakeup(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

/*******************************************************************************
* @brief      Watchdog interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void WDT_Handler_TestSleepAndWdtWakeup(void) {
	uint32_t status;

	status = HAL_WDT_GetStatus();						// Get WDT Status
	WakeupFlag = HAL_SCU_GetWakeUpSRCStatus();			// Get Wakeup Status

	if (status & WDT_SR_UNFIFLAG) {  				// UNFIFLAG
		_DBG("Under-Flow Occurred!\r\n");
		if (wdtf_uf == 0){
			wdtf_uf = 1;
			HAL_GPIO_ClearPin(PE, _BIT(0));
		} else {
			wdtf_uf = 0;
			HAL_GPIO_SetPin(PE, _BIT(0));
		}
		HAL_WDT_ClearStatus(WDT_STATUS_UNDERFLOW_INT);
	}

	if (status & WDT_SR_WINMIFLAG) { 				// WINMIFLAG
		_DBG("Window Match Occurred!\r\n");

		if (wdtf_wmf == 0){
			wdtf_wmf = 1;
			HAL_GPIO_ClearPin(PE, _BIT(1));
		} else {
			wdtf_wmf = 0;
			HAL_GPIO_SetPin(PE, _BIT(1));
		}
		HAL_WDT_ClearStatus(WDT_STATUS_WINDOW_MATCH_INT);
	}

	gCnt++;
	if(gCnt == 10) {
		_DBG("WDT STOP \r\n");
		HAL_WDT_DeInit();
	}
}

/*******************************************************************************
* Public Function
*******************************************************************************/

/**********************************************************************
 * @brief		Test Deep-sleep mode and WDT wakeup
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Main_TestSleepAndWdtWakeup(void) {
	uint32_t	PrevMode;
	WDT_CFG_Type WdtCfg;
    int i;

	// LED blink pin (PE0)
	HAL_GPIO_ConfigOutput(PE, 0, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 0, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(0));

	// Clock Output Pin setting (PF4)
	HAL_GPIO_ConfigFunction((PORT_Type *)PF, 4, PCU_ALT_FUNCTION_1);
	HAL_GPIO_ConfigOutput((PORT_Type *)PF, 4, PCU_MODE_ALT_FUNC);

	HAL_SCU_ClockOutput(4,ENABLE); // CLKO Freq. = Divided PCLK/((1+4)+*2)

	// Disable SysTick
	SysTick->CTRL &= ~SysTick_CTRL_ENABLE_Msk;

	HAL_SCU_LSI_ClockConfig(LSI_EN);						// LSI (500kHz) Enable for FPGA Test, (LSI clock is always on.)
	HAL_SCU_WDT_ClockConfig(WDTCLK_WDTRC);					// 31250Hz clock as watch-dog clock
	WdtCfg.EnableReset = FALSE;							// WDT Reset Disable
	WdtCfg.ClockDivider = WDT_DIVIDER_4;  			// WDTCLK = WDTCLKIN / PRS = 7812Hz
	WdtCfg.Data = (78120/2) - 1; 							// 5s @ 7812Hz
	HAL_WDT_Init(&WdtCfg);

	cputs("Enter Sleep... ZzZzZz...");

    for (i = 0; i < 0x10000; i++);  // delay

	HAL_SCU_SystemClockChange(SCCR_LSI);		// Change Clock to LSI(500kHz) to reduce power consumption
	HAL_SCU_LSE_ClockConfig(DISABLE);
	HAL_SCU_HSI_ClockConfig(DISABLE);
	HAL_SCU_HSI_ClockConfig(DISABLE);

	HAL_SCU_WakeUpSRCCmd(WAKEUP_WDT, ENABLE);			// Enable Wake-up Source
	HAL_WDT_ConfigureInterrupt(WDT_INTCFG_WINMI_DISABLE, WDT_INTCFG_UNFI_ENABLE);		// Enable Underflow Interrupt for Wake-up
    NVIC_SetPriority(WDT_IRQn, 3);
    NVIC_EnableIRQ(WDT_IRQn);
	__enable_irq();

    HAL_PWR_AlwaysOnLSIForDeepSleep(LSIAON_ENABLE, BGRAON_DISABLE, VDCAON_DISABLE);	// LSI always-on for WDT wakeup

	HAL_WDT_Start();					// Start WDT operation

	HAL_PWR_EnterSleep();			// go to sleep mode

	HAL_WDT_DeInit();
	HAL_SCU_WakeUpSRCCmd(WAKEUP_WDT, DISABLE);			// Disable Wake-up Source
	HAL_WDT_ConfigureInterrupt(WDT_INTCFG_WINMI_DISABLE, WDT_INTCFG_UNFI_DISABLE);
	NVIC_DisableIRQ(WDT_IRQn);
	NVIC_ClearPendingIRQ(WDT_IRQn);								// Clear WDTInterrupt flag

#if 1
    HAL_SCU_SetPLLFreq(IRC32MHz, PLL32MHz);     // Use PLL with HSI clock
#else
    HAL_SCU_SetPLLFreq(XTAL8MHz, PLL48MHz);   // Use PLL with HSE clock (Initialization System Clock)
#endif
	HAL_SCU_ClockOutput(4,ENABLE);

    debug_frmwrk_init();

    for (i = 0; i < 0x10000; i++);  // delay

	SysTick_Config(SystemCoreClock/1000);  						// 1ms interrupt

	if ( (WakeupFlag & WAKEUP_WDT) == WAKEUP_WDT) {
		cputs("Waked up from Sleep!\r\n");
	}

	PrevMode = HAL_SCU_GetPreviousModeBeforeResetEvent();
	if(PrevMode == PREMODE_SLEEP) {
		_DBG("Previous Mode is Sleep\r\n");
    }

	while(1) {
		MilliSec = 500;
		while(MilliSec);
		HAL_GPIO_SetPin(PE, _BIT(0));
		MilliSec = 500;
		while(MilliSec);
		HAL_GPIO_ClearPin(PE, _BIT(0));
	}
}

/* --------------------------------- End Of File ------------------------------ */
