/*****************************************************************************
* @file     TestCode_WT_Interrupt.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_WT_DeepSleepWakeup.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint8_t WtFlag;  			// underflow flag
static uint32_t WakeupFlag;         // wakeup flag
static uint32_t MilliSec;	        // Systick Counter Value

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_WtDeepSleepWakeup(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

/*******************************************************************************
* @brief      Watch Timer interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void WT_IRQHandler_WtDeepSleepWakeup(void) {
	FlagStatus status;

	status = HAL_WT_GetStatus();				// Get WT Interrupt Status
	WakeupFlag = HAL_SCU_GetWakeUpSRCStatus();			// Get Wakeup Status

	if (status){  									// WT Interrupt Flag
		cputs("Watch-Timer Interrupt Occurred!\r\n");
		if (WtFlag == 0){
			WtFlag = 1;
			HAL_GPIO_ClearPin(PE, _BIT(0));
		} else {
			WtFlag = 0;
			HAL_GPIO_SetPin(PE, _BIT(0));
		}
		HAL_WT_ClearStatus();
	}
}

/*******************************************************************************
* @brief      Test Deep Sleep mode
* @param      None
* @return     None
*******************************************************************************/
void Main_TestDeepSleepAndWtWakeup(void) {
	uint32_t	PrevMode;
	WT_CFG_Type WtCfg;
    int i;

	// LED blink pin (PE0)
	HAL_GPIO_ConfigOutput(PE, 0, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 0, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(0));

	// Clock Output Pin setting (PF4)
	HAL_GPIO_ConfigFunction((PORT_Type *)PF, 4, PCU_ALT_FUNCTION_1);
	HAL_GPIO_ConfigOutput((PORT_Type *)PF, 4, PCU_MODE_ALT_FUNC);
	HAL_SCU_ClockOutput(4,ENABLE);									// CLKO Freq. = Divided PCLK/((1+4)+*2)

	SysTick->CTRL &= ~SysTick_CTRL_ENABLE_Msk;				// Disable Systick

	HAL_SCU_LSI_ClockConfig(LSI_EN);										// LSI (500kHz) Enable for FPGA Test, (LSI clock is always on.)
	HAL_SCU_WT_ClockConfig(WTCLK_WDTRC);								// 31250Hz clock as Watch-Timer clock

	WtCfg.ClockDivider = WT_DIVIDER_2_14_MUL_WTDR;  		// WTCLK = WTCLKIN / PRS = 7812Hz
	WtCfg.MatchData = 5; 									// WT Value
	HAL_WT_Init(&WtCfg);										// WT Configuration Setting

	cputs("Enter Deep Sleep... ZzZzZz...\r\n");

    for (i = 0; i < 0x10000; i++);  // delay

	HAL_SCU_SystemClockChange(SCCR_LSI);								// Change Clock to LSI(500kHz) to reduce power consumption
	HAL_SCU_LSE_ClockConfig(DISABLE);
	HAL_SCU_HSI_ClockConfig(DISABLE);
	HAL_SCU_HSI_ClockConfig(DISABLE);

	HAL_SCU_WakeUpSRCCmd(WAKEUP_WT, ENABLE);					// Enable Wake-up Source
	HAL_WT_ConfigInterrupt(ENABLE);							// Enable WT Interrupt for Wake-up
    NVIC_SetPriority(WT_IRQn, 3);
    NVIC_EnableIRQ(WT_IRQn);
	__enable_irq();

	HAL_WT_Start();												// Start WT operation

	HAL_PWR_AlwaysOnLSIForDeepSleep(LSIAON_ENABLE, BGRAON_DISABLE, VDCAON_DISABLE);	// LSI always-on for deep-sleep
	HAL_PWR_EnterDeepSleep();									// go to deep-sleep mode

	HAL_WT_Stop();
	HAL_SCU_WakeUpSRCCmd(WAKEUP_WT, DISABLE);					// Disable Wake-up Source
	HAL_WT_ConfigInterrupt(DISABLE);							// Disable WT Interrupt for Wake-up
	NVIC_DisableIRQ(WT_IRQn);
	NVIC_ClearPendingIRQ(WT_IRQn);							    // Clear WTInterrupt flag

#if 1
    HAL_SCU_SetPLLFreq(IRC32MHz, PLL32MHz);     // Use PLL with HSI clock
#else
    HAL_SCU_SetPLLFreq(XTAL8MHz, PLL48MHz);   // Use PLL with HSE clock (Initialization System Clock)
#endif

    debug_frmwrk_init();
    
    for (i = 0; i < 0x10000; i++);  // delay    

	HAL_SCU_ClockOutput(4,ENABLE);

	SysTick_Config(SystemCoreClock/1000);  					// 1ms interrupt

	if ( (WakeupFlag&WAKEUP_WT) == WAKEUP_WT) {				// Check Wake-up Status
		cputs("Waked up from Deep-Sleep!\r\n");
	}

	PrevMode = HAL_SCU_GetPreviousModeBeforeResetEvent();		// Check Previous mode
	if(PrevMode == PREMODE_DEEPSLEEP)
		cputs("Previous Mode is Deep-Sleep\r\n");

	while(1) {												// Blink LED (PE0)
		MilliSec = 500;
		while(MilliSec);
		HAL_GPIO_SetPin(PE, _BIT(0));
		MilliSec = 500;
		while(MilliSec);
		HAL_GPIO_ClearPin(PE, _BIT(0));
	}
}

/* --------------------------------- End Of File ------------------------------ */
